(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 3.0,
MathReader 3.0, or any compatible application. The data for the notebook 
starts with the line of stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[    104886,       2784]*)
(*NotebookOutlinePosition[    133494,       3828]*)
(*  CellTagsIndexPosition[    133450,       3824]*)
(*WindowFrame->Normal*)



Notebook[{
Cell[TextData[{
  "\n\n\nArray Processing Idioms\n\n\n",
  StyleBox["Mark D. Reeve\n\nXonTech, Inc.\n\n\n\nWorldwide ",
    FontSize->18],
  StyleBox["Mathematica",
    FontSize->18,
    FontSlant->"Italic"],
  StyleBox[" Conference\nJune 20, 1998\n\n\n\n\n\n",
    FontSize->18]
}], "Title",
  TextAlignment->Center],

Cell["\<\

  
\
\>", "Text"],

Cell["Introduction", "Subtitle"],

Cell["\<\
Ubiquity of Arrays
\t- Multi-dimensional arrays\
\>", "Exercise"],

Cell["\<\
Need to Interactively Analyze, Manipulate, and Visualize Data
\t- Go beyond canned packages.
\t- Free the user from the need to spend most of the time developing code.\
\>", "Exercise"],

Cell[TextData[{
  "Mathematica\n\t- Fits the bill,\n\t- Yet demands an array-based mindset to \
be used efficiently.\n\t- Many users, under deadline pressures, revert to \
FORTRAN / C coding paradigms\n\t\t* Horribly inefficient to execute\n\t\t* \
Slower to code and debug\n\t\t* Difficult to maintain\n\t\t* Cloak the simple \
structure of a problem in a bulky and artificial formalism\n\t\t",
  StyleBox["* Self-perpetuating )-:",
    FontColor->RGBColor[1, 0, 1]]
}], "Exercise"],

Cell[TextData[{
  "An Idiomatic Approach\n\t- Develop a store of handy idioms\n\t- Decompose \
problems into chunks to solve (efficiently) with one or two idioms\n\t- \
String the idioms together to instantly get concise, easily maintained \
analysis code\n\t\t",
  StyleBox["* Self-perpetuating :-)",
    FontColor->RGBColor[1, 0, 1]]
}], "Exercise"],

Cell[CellGroupData[{

Cell["Quick Example", "Section"],

Cell["Launch kernel and eliminate spell warnings.", "Text"],

Cell["Off[General::spell1,General::spell2,General::spell3]", "Input"],

Cell["Given a 5-D array, sum over the last two dimensions.", "Text"],

Cell[CellGroupData[{

Cell["\<\
arr5d=Fold[Partition,Range[720],{6,5,4,3}];
Dimensions[arr5d]\
\>", "Input"],

Cell[BoxData[
    \({2, 3, 4, 5, 6}\)], "Output"]
}, Open  ]],

Cell["The traditional (hard) way.", "Text"],

Cell[CellGroupData[{

Cell["\<\
hard=Table[0,{2},{3},{4}];
Do[
  Do[
    Do[
      Do[
        Do[
          hard[[i,j,k]]+=arr5d[[i,j,k,m,n]];
        ,{n,1,6}];
      ,{m,1,5}];
    ,{k,1,4}];
  ,{j,1,3}];
,{i,1,2}]; // Timing

Dimensions[hard]\
\>", "Input"],

Cell[BoxData[
    \({0.110000000000013642`\ Second, Null}\)], "Output"],

Cell[BoxData[
    \({2, 3, 4}\)], "Output"]
}, Open  ]],

Cell["The array-based (easy) way.", "Text"],

Cell[CellGroupData[{

Cell["\<\
easy = Apply[Plus,arr5d,{3,4}]; // Timing
Dimensions[easy]\
\>", "Input"],

Cell[BoxData[
    \({0.`\ Second, Null}\)], "Output"],

Cell[BoxData[
    \({2, 3, 4}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["easy==hard   (* The results are the same, of course *)", "Input"],

Cell[BoxData[
    \(True\)], "Output"]
}, Open  ]],

Cell["\<\

 \
\>", "Text"],

Cell["Fundamental Constructs", "Subtitle"],

Cell["\<\

\
\>", "Text"],

Cell[CellGroupData[{

Cell["Lists and Arrays", "Subsection"],

Cell[TextData[{
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " stores arrays in lists.  Any item in a list can be another list, or \
anything else.  2-D arrays are lists of lists.  The most important thing to \
remember about lists is:\n\t\t\t",
  StyleBox["THE FIRST DIMENSION IS MORE EQUAL THAN OTHERS",
    FontWeight->"Bold"],
  "\nIndexing over the first dimension of a list gives you everything below \
it."
}], "Text",
  TextAlignment->Left],

Cell[CellGroupData[{

Cell["\<\
Dimensions[arr5d]    (* recall that we have this 5-D array defined *)\
\>", "Input"],

Cell[BoxData[
    \({2, 3, 4, 5, 6}\)], "Output"]
}, Open  ]],

Cell["Note that this is really 2 arrays, each of size 3x4x5x6.", "Text"],

Cell[CellGroupData[{

Cell["\<\
(* Extract the second item of the outer list, and the third of next one in *)
Dimensions[ arr5d[[2,3]] ]  \
\>", "Input"],

Cell[BoxData[
    \({4, 5, 6}\)], "Output"]
}, Open  ]],

Cell["\<\
For ease of display, let's work with the first level for a moment.\
\>", "Text"],

Cell[CellGroupData[{

Cell["\<\
arr4d=arr5d[[1]];
Dimensions[arr4d]\
\>", "Input"],

Cell[BoxData[
    \({3, 4, 5, 6}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MatrixForm[arr4d]", "Input"],

Cell[BoxData[
    InterpretationBox[
      RowBox[{"(", GridBox[{
            {
              RowBox[{"(", GridBox[{
                    {"1", "2", "3", "4", "5", "6"},
                    {"7", "8", "9", "10", "11", "12"},
                    {"13", "14", "15", "16", "17", "18"},
                    {"19", "20", "21", "22", "23", "24"},
                    {"25", "26", "27", "28", "29", "30"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"31", "32", "33", "34", "35", "36"},
                    {"37", "38", "39", "40", "41", "42"},
                    {"43", "44", "45", "46", "47", "48"},
                    {"49", "50", "51", "52", "53", "54"},
                    {"55", "56", "57", "58", "59", "60"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"61", "62", "63", "64", "65", "66"},
                    {"67", "68", "69", "70", "71", "72"},
                    {"73", "74", "75", "76", "77", "78"},
                    {"79", "80", "81", "82", "83", "84"},
                    {"85", "86", "87", "88", "89", "90"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"91", "92", "93", "94", "95", "96"},
                    {"97", "98", "99", "100", "101", "102"},
                    {"103", "104", "105", "106", "107", "108"},
                    {"109", "110", "111", "112", "113", "114"},
                    {"115", "116", "117", "118", "119", "120"}
                    }], ")"}]},
            {
              RowBox[{"(", GridBox[{
                    {"121", "122", "123", "124", "125", "126"},
                    {"127", "128", "129", "130", "131", "132"},
                    {"133", "134", "135", "136", "137", "138"},
                    {"139", "140", "141", "142", "143", "144"},
                    {"145", "146", "147", "148", "149", "150"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"151", "152", "153", "154", "155", "156"},
                    {"157", "158", "159", "160", "161", "162"},
                    {"163", "164", "165", "166", "167", "168"},
                    {"169", "170", "171", "172", "173", "174"},
                    {"175", "176", "177", "178", "179", "180"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"181", "182", "183", "184", "185", "186"},
                    {"187", "188", "189", "190", "191", "192"},
                    {"193", "194", "195", "196", "197", "198"},
                    {"199", "200", "201", "202", "203", "204"},
                    {"205", "206", "207", "208", "209", "210"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"211", "212", "213", "214", "215", "216"},
                    {"217", "218", "219", "220", "221", "222"},
                    {"223", "224", "225", "226", "227", "228"},
                    {"229", "230", "231", "232", "233", "234"},
                    {"235", "236", "237", "238", "239", "240"}
                    }], ")"}]},
            {
              RowBox[{"(", GridBox[{
                    {"241", "242", "243", "244", "245", "246"},
                    {"247", "248", "249", "250", "251", "252"},
                    {"253", "254", "255", "256", "257", "258"},
                    {"259", "260", "261", "262", "263", "264"},
                    {"265", "266", "267", "268", "269", "270"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"271", "272", "273", "274", "275", "276"},
                    {"277", "278", "279", "280", "281", "282"},
                    {"283", "284", "285", "286", "287", "288"},
                    {"289", "290", "291", "292", "293", "294"},
                    {"295", "296", "297", "298", "299", "300"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"301", "302", "303", "304", "305", "306"},
                    {"307", "308", "309", "310", "311", "312"},
                    {"313", "314", "315", "316", "317", "318"},
                    {"319", "320", "321", "322", "323", "324"},
                    {"325", "326", "327", "328", "329", "330"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"331", "332", "333", "334", "335", "336"},
                    {"337", "338", "339", "340", "341", "342"},
                    {"343", "344", "345", "346", "347", "348"},
                    {"349", "350", "351", "352", "353", "354"},
                    {"355", "356", "357", "358", "359", "360"}
                    }], ")"}]}
            }], ")"}],
      MatrixForm[ {{{{1, 2, 3, 4, 5, 6}, {7, 8, 9, 10, 11, 12}, {13, 14, 15, 
        16, 17, 18}, {19, 20, 21, 22, 23, 24}, {25, 26, 27, 28, 29, 30}}, {{
        31, 32, 33, 34, 35, 36}, {37, 38, 39, 40, 41, 42}, {43, 44, 45, 46, 
        47, 48}, {49, 50, 51, 52, 53, 54}, {55, 56, 57, 58, 59, 60}}, {{61, 
        62, 63, 64, 65, 66}, {67, 68, 69, 70, 71, 72}, {73, 74, 75, 76, 77, 
        78}, {79, 80, 81, 82, 83, 84}, {85, 86, 87, 88, 89, 90}}, {{91, 92, 
        93, 94, 95, 96}, {97, 98, 99, 100, 101, 102}, {103, 104, 105, 106, 
        107, 108}, {109, 110, 111, 112, 113, 114}, {115, 116, 117, 118, 119, 
        120}}}, {{{121, 122, 123, 124, 125, 126}, {127, 128, 129, 130, 131, 
        132}, {133, 134, 135, 136, 137, 138}, {139, 140, 141, 142, 143, 
        144}, {145, 146, 147, 148, 149, 150}}, {{151, 152, 153, 154, 155, 
        156}, {157, 158, 159, 160, 161, 162}, {163, 164, 165, 166, 167, 
        168}, {169, 170, 171, 172, 173, 174}, {175, 176, 177, 178, 179, 
        180}}, {{181, 182, 183, 184, 185, 186}, {187, 188, 189, 190, 191, 
        192}, {193, 194, 195, 196, 197, 198}, {199, 200, 201, 202, 203, 
        204}, {205, 206, 207, 208, 209, 210}}, {{211, 212, 213, 214, 215, 
        216}, {217, 218, 219, 220, 221, 222}, {223, 224, 225, 226, 227, 
        228}, {229, 230, 231, 232, 233, 234}, {235, 236, 237, 238, 239, 
        240}}}, {{{241, 242, 243, 244, 245, 246}, {247, 248, 249, 250, 251, 
        252}, {253, 254, 255, 256, 257, 258}, {259, 260, 261, 262, 263, 
        264}, {265, 266, 267, 268, 269, 270}}, {{271, 272, 273, 274, 275, 
        276}, {277, 278, 279, 280, 281, 282}, {283, 284, 285, 286, 287, 
        288}, {289, 290, 291, 292, 293, 294}, {295, 296, 297, 298, 299, 
        300}}, {{301, 302, 303, 304, 305, 306}, {307, 308, 309, 310, 311, 
        312}, {313, 314, 315, 316, 317, 318}, {319, 320, 321, 322, 323, 
        324}, {325, 326, 327, 328, 329, 330}}, {{331, 332, 333, 334, 335, 
        336}, {337, 338, 339, 340, 341, 342}, {343, 344, 345, 346, 347, 
        348}, {349, 350, 351, 352, 353, 354}, {355, 356, 357, 358, 359, 
        360}}}}]]], "Output"]
}, Open  ]],

Cell["\<\
It can be extremely handy to work on the first dimension.  For example, to \
add 1 to the first outer row, 2 to the second, and 3 to third, do this:\
\>", "Text"],

Cell[CellGroupData[{

Cell["\<\
arr4d + {1,2,3} // MatrixForm  (* That's all there is to it!  No messy loops! \
*)\
\>", "Input"],

Cell[BoxData[
    InterpretationBox[
      RowBox[{"(", GridBox[{
            {
              RowBox[{"(", GridBox[{
                    {"2", "3", "4", "5", "6", "7"},
                    {"8", "9", "10", "11", "12", "13"},
                    {"14", "15", "16", "17", "18", "19"},
                    {"20", "21", "22", "23", "24", "25"},
                    {"26", "27", "28", "29", "30", "31"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"32", "33", "34", "35", "36", "37"},
                    {"38", "39", "40", "41", "42", "43"},
                    {"44", "45", "46", "47", "48", "49"},
                    {"50", "51", "52", "53", "54", "55"},
                    {"56", "57", "58", "59", "60", "61"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"62", "63", "64", "65", "66", "67"},
                    {"68", "69", "70", "71", "72", "73"},
                    {"74", "75", "76", "77", "78", "79"},
                    {"80", "81", "82", "83", "84", "85"},
                    {"86", "87", "88", "89", "90", "91"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"92", "93", "94", "95", "96", "97"},
                    {"98", "99", "100", "101", "102", "103"},
                    {"104", "105", "106", "107", "108", "109"},
                    {"110", "111", "112", "113", "114", "115"},
                    {"116", "117", "118", "119", "120", "121"}
                    }], ")"}]},
            {
              RowBox[{"(", GridBox[{
                    {"123", "124", "125", "126", "127", "128"},
                    {"129", "130", "131", "132", "133", "134"},
                    {"135", "136", "137", "138", "139", "140"},
                    {"141", "142", "143", "144", "145", "146"},
                    {"147", "148", "149", "150", "151", "152"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"153", "154", "155", "156", "157", "158"},
                    {"159", "160", "161", "162", "163", "164"},
                    {"165", "166", "167", "168", "169", "170"},
                    {"171", "172", "173", "174", "175", "176"},
                    {"177", "178", "179", "180", "181", "182"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"183", "184", "185", "186", "187", "188"},
                    {"189", "190", "191", "192", "193", "194"},
                    {"195", "196", "197", "198", "199", "200"},
                    {"201", "202", "203", "204", "205", "206"},
                    {"207", "208", "209", "210", "211", "212"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"213", "214", "215", "216", "217", "218"},
                    {"219", "220", "221", "222", "223", "224"},
                    {"225", "226", "227", "228", "229", "230"},
                    {"231", "232", "233", "234", "235", "236"},
                    {"237", "238", "239", "240", "241", "242"}
                    }], ")"}]},
            {
              RowBox[{"(", GridBox[{
                    {"244", "245", "246", "247", "248", "249"},
                    {"250", "251", "252", "253", "254", "255"},
                    {"256", "257", "258", "259", "260", "261"},
                    {"262", "263", "264", "265", "266", "267"},
                    {"268", "269", "270", "271", "272", "273"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"274", "275", "276", "277", "278", "279"},
                    {"280", "281", "282", "283", "284", "285"},
                    {"286", "287", "288", "289", "290", "291"},
                    {"292", "293", "294", "295", "296", "297"},
                    {"298", "299", "300", "301", "302", "303"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"304", "305", "306", "307", "308", "309"},
                    {"310", "311", "312", "313", "314", "315"},
                    {"316", "317", "318", "319", "320", "321"},
                    {"322", "323", "324", "325", "326", "327"},
                    {"328", "329", "330", "331", "332", "333"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"334", "335", "336", "337", "338", "339"},
                    {"340", "341", "342", "343", "344", "345"},
                    {"346", "347", "348", "349", "350", "351"},
                    {"352", "353", "354", "355", "356", "357"},
                    {"358", "359", "360", "361", "362", "363"}
                    }], ")"}]}
            }], ")"}],
      MatrixForm[ {{{{2, 3, 4, 5, 6, 7}, {8, 9, 10, 11, 12, 13}, {14, 15, 16, 
        17, 18, 19}, {20, 21, 22, 23, 24, 25}, {26, 27, 28, 29, 30, 31}}, {{
        32, 33, 34, 35, 36, 37}, {38, 39, 40, 41, 42, 43}, {44, 45, 46, 47, 
        48, 49}, {50, 51, 52, 53, 54, 55}, {56, 57, 58, 59, 60, 61}}, {{62, 
        63, 64, 65, 66, 67}, {68, 69, 70, 71, 72, 73}, {74, 75, 76, 77, 78, 
        79}, {80, 81, 82, 83, 84, 85}, {86, 87, 88, 89, 90, 91}}, {{92, 93, 
        94, 95, 96, 97}, {98, 99, 100, 101, 102, 103}, {104, 105, 106, 107, 
        108, 109}, {110, 111, 112, 113, 114, 115}, {116, 117, 118, 119, 120, 
        121}}}, {{{123, 124, 125, 126, 127, 128}, {129, 130, 131, 132, 133, 
        134}, {135, 136, 137, 138, 139, 140}, {141, 142, 143, 144, 145, 
        146}, {147, 148, 149, 150, 151, 152}}, {{153, 154, 155, 156, 157, 
        158}, {159, 160, 161, 162, 163, 164}, {165, 166, 167, 168, 169, 
        170}, {171, 172, 173, 174, 175, 176}, {177, 178, 179, 180, 181, 
        182}}, {{183, 184, 185, 186, 187, 188}, {189, 190, 191, 192, 193, 
        194}, {195, 196, 197, 198, 199, 200}, {201, 202, 203, 204, 205, 
        206}, {207, 208, 209, 210, 211, 212}}, {{213, 214, 215, 216, 217, 
        218}, {219, 220, 221, 222, 223, 224}, {225, 226, 227, 228, 229, 
        230}, {231, 232, 233, 234, 235, 236}, {237, 238, 239, 240, 241, 
        242}}}, {{{244, 245, 246, 247, 248, 249}, {250, 251, 252, 253, 254, 
        255}, {256, 257, 258, 259, 260, 261}, {262, 263, 264, 265, 266, 
        267}, {268, 269, 270, 271, 272, 273}}, {{274, 275, 276, 277, 278, 
        279}, {280, 281, 282, 283, 284, 285}, {286, 287, 288, 289, 290, 
        291}, {292, 293, 294, 295, 296, 297}, {298, 299, 300, 301, 302, 
        303}}, {{304, 305, 306, 307, 308, 309}, {310, 311, 312, 313, 314, 
        315}, {316, 317, 318, 319, 320, 321}, {322, 323, 324, 325, 326, 
        327}, {328, 329, 330, 331, 332, 333}}, {{334, 335, 336, 337, 338, 
        339}, {340, 341, 342, 343, 344, 345}, {346, 347, 348, 349, 350, 
        351}, {352, 353, 354, 355, 356, 357}, {358, 359, 360, 361, 362, 
        363}}}}]]], "Output"]
}, Open  ]],

Cell["\<\
However, if you want to do something to the second LEVEL, you have to get \
past that pesky first dimension.\
\>", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Map", "Subsection"],

Cell[TextData[{
  "Arguably the most useful operator in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ", Map allows you to target arbitrary levels of a list.  Just give it the \
function you want to operate with, the data to operate on, the level at which \
to operate, and you're done:"
}], "Text"],

Cell[CellGroupData[{

Cell["\<\
Map[ Range[4]+#&, arr4d, {1}] // MatrixForm  
(* Note the pure function, and that the level is 1 *)\
\>", "Input"],

Cell[BoxData[
    InterpretationBox[
      RowBox[{"(", GridBox[{
            {
              RowBox[{"(", GridBox[{
                    {"2", "3", "4", "5", "6", "7"},
                    {"8", "9", "10", "11", "12", "13"},
                    {"14", "15", "16", "17", "18", "19"},
                    {"20", "21", "22", "23", "24", "25"},
                    {"26", "27", "28", "29", "30", "31"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"33", "34", "35", "36", "37", "38"},
                    {"39", "40", "41", "42", "43", "44"},
                    {"45", "46", "47", "48", "49", "50"},
                    {"51", "52", "53", "54", "55", "56"},
                    {"57", "58", "59", "60", "61", "62"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"64", "65", "66", "67", "68", "69"},
                    {"70", "71", "72", "73", "74", "75"},
                    {"76", "77", "78", "79", "80", "81"},
                    {"82", "83", "84", "85", "86", "87"},
                    {"88", "89", "90", "91", "92", "93"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"95", "96", "97", "98", "99", "100"},
                    {"101", "102", "103", "104", "105", "106"},
                    {"107", "108", "109", "110", "111", "112"},
                    {"113", "114", "115", "116", "117", "118"},
                    {"119", "120", "121", "122", "123", "124"}
                    }], ")"}]},
            {
              RowBox[{"(", GridBox[{
                    {"122", "123", "124", "125", "126", "127"},
                    {"128", "129", "130", "131", "132", "133"},
                    {"134", "135", "136", "137", "138", "139"},
                    {"140", "141", "142", "143", "144", "145"},
                    {"146", "147", "148", "149", "150", "151"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"153", "154", "155", "156", "157", "158"},
                    {"159", "160", "161", "162", "163", "164"},
                    {"165", "166", "167", "168", "169", "170"},
                    {"171", "172", "173", "174", "175", "176"},
                    {"177", "178", "179", "180", "181", "182"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"184", "185", "186", "187", "188", "189"},
                    {"190", "191", "192", "193", "194", "195"},
                    {"196", "197", "198", "199", "200", "201"},
                    {"202", "203", "204", "205", "206", "207"},
                    {"208", "209", "210", "211", "212", "213"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"215", "216", "217", "218", "219", "220"},
                    {"221", "222", "223", "224", "225", "226"},
                    {"227", "228", "229", "230", "231", "232"},
                    {"233", "234", "235", "236", "237", "238"},
                    {"239", "240", "241", "242", "243", "244"}
                    }], ")"}]},
            {
              RowBox[{"(", GridBox[{
                    {"242", "243", "244", "245", "246", "247"},
                    {"248", "249", "250", "251", "252", "253"},
                    {"254", "255", "256", "257", "258", "259"},
                    {"260", "261", "262", "263", "264", "265"},
                    {"266", "267", "268", "269", "270", "271"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"273", "274", "275", "276", "277", "278"},
                    {"279", "280", "281", "282", "283", "284"},
                    {"285", "286", "287", "288", "289", "290"},
                    {"291", "292", "293", "294", "295", "296"},
                    {"297", "298", "299", "300", "301", "302"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"304", "305", "306", "307", "308", "309"},
                    {"310", "311", "312", "313", "314", "315"},
                    {"316", "317", "318", "319", "320", "321"},
                    {"322", "323", "324", "325", "326", "327"},
                    {"328", "329", "330", "331", "332", "333"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"335", "336", "337", "338", "339", "340"},
                    {"341", "342", "343", "344", "345", "346"},
                    {"347", "348", "349", "350", "351", "352"},
                    {"353", "354", "355", "356", "357", "358"},
                    {"359", "360", "361", "362", "363", "364"}
                    }], ")"}]}
            }], ")"}],
      MatrixForm[ {{{{2, 3, 4, 5, 6, 7}, {8, 9, 10, 11, 12, 13}, {14, 15, 16, 
        17, 18, 19}, {20, 21, 22, 23, 24, 25}, {26, 27, 28, 29, 30, 31}}, {{
        33, 34, 35, 36, 37, 38}, {39, 40, 41, 42, 43, 44}, {45, 46, 47, 48, 
        49, 50}, {51, 52, 53, 54, 55, 56}, {57, 58, 59, 60, 61, 62}}, {{64, 
        65, 66, 67, 68, 69}, {70, 71, 72, 73, 74, 75}, {76, 77, 78, 79, 80, 
        81}, {82, 83, 84, 85, 86, 87}, {88, 89, 90, 91, 92, 93}}, {{95, 96, 
        97, 98, 99, 100}, {101, 102, 103, 104, 105, 106}, {107, 108, 109, 
        110, 111, 112}, {113, 114, 115, 116, 117, 118}, {119, 120, 121, 122, 
        123, 124}}}, {{{122, 123, 124, 125, 126, 127}, {128, 129, 130, 131, 
        132, 133}, {134, 135, 136, 137, 138, 139}, {140, 141, 142, 143, 144, 
        145}, {146, 147, 148, 149, 150, 151}}, {{153, 154, 155, 156, 157, 
        158}, {159, 160, 161, 162, 163, 164}, {165, 166, 167, 168, 169, 
        170}, {171, 172, 173, 174, 175, 176}, {177, 178, 179, 180, 181, 
        182}}, {{184, 185, 186, 187, 188, 189}, {190, 191, 192, 193, 194, 
        195}, {196, 197, 198, 199, 200, 201}, {202, 203, 204, 205, 206, 
        207}, {208, 209, 210, 211, 212, 213}}, {{215, 216, 217, 218, 219, 
        220}, {221, 222, 223, 224, 225, 226}, {227, 228, 229, 230, 231, 
        232}, {233, 234, 235, 236, 237, 238}, {239, 240, 241, 242, 243, 
        244}}}, {{{242, 243, 244, 245, 246, 247}, {248, 249, 250, 251, 252, 
        253}, {254, 255, 256, 257, 258, 259}, {260, 261, 262, 263, 264, 
        265}, {266, 267, 268, 269, 270, 271}}, {{273, 274, 275, 276, 277, 
        278}, {279, 280, 281, 282, 283, 284}, {285, 286, 287, 288, 289, 
        290}, {291, 292, 293, 294, 295, 296}, {297, 298, 299, 300, 301, 
        302}}, {{304, 305, 306, 307, 308, 309}, {310, 311, 312, 313, 314, 
        315}, {316, 317, 318, 319, 320, 321}, {322, 323, 324, 325, 326, 
        327}, {328, 329, 330, 331, 332, 333}}, {{335, 336, 337, 338, 339, 
        340}, {341, 342, 343, 344, 345, 346}, {347, 348, 349, 350, 351, 
        352}, {353, 354, 355, 356, 357, 358}, {359, 360, 361, 362, 363, 
        364}}}}]]], "Output"]
}, Open  ]],

Cell[TextData[{
  StyleBox["\nPURE FUNCTIONS:  ",
    FontWeight->"Bold"],
  "These are just easier versions of normal functions.  Range[4]+#& is \
equivalent to Add4X[arr_]:=Range[4]+arr.\nWe'll use them frequently from here \
on out.  They avoid needlessly defining hundreds of never-to-be-used-again \
functions.\n"
}], "Text"],

Cell["\<\
The default level for Map is {1}, so there's a shortand for it that you see \
everywhere:\
\>", "Text"],

Cell["Range[4]|#& /@ arr4d;  (* does the same thing *)", "Input"],

Cell[TextData[{
  "To get further down, you just increment the level.  Note that the \
specified level is the number of dimensions you want to Map ",
  StyleBox["over",
    FontSlant->"Italic"],
  "."
}], "Text"],

Cell[CellGroupData[{

Cell["\<\
Map[Range[6]+#&,arr4d,{3}] // MatrixForm  (* here I've added 1-6 over the \
last dimension *)\
\>", "Input"],

Cell[BoxData[
    InterpretationBox[
      RowBox[{"(", GridBox[{
            {
              RowBox[{"(", GridBox[{
                    {"2", "4", "6", "8", "10", "12"},
                    {"8", "10", "12", "14", "16", "18"},
                    {"14", "16", "18", "20", "22", "24"},
                    {"20", "22", "24", "26", "28", "30"},
                    {"26", "28", "30", "32", "34", "36"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"32", "34", "36", "38", "40", "42"},
                    {"38", "40", "42", "44", "46", "48"},
                    {"44", "46", "48", "50", "52", "54"},
                    {"50", "52", "54", "56", "58", "60"},
                    {"56", "58", "60", "62", "64", "66"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"62", "64", "66", "68", "70", "72"},
                    {"68", "70", "72", "74", "76", "78"},
                    {"74", "76", "78", "80", "82", "84"},
                    {"80", "82", "84", "86", "88", "90"},
                    {"86", "88", "90", "92", "94", "96"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"92", "94", "96", "98", "100", "102"},
                    {"98", "100", "102", "104", "106", "108"},
                    {"104", "106", "108", "110", "112", "114"},
                    {"110", "112", "114", "116", "118", "120"},
                    {"116", "118", "120", "122", "124", "126"}
                    }], ")"}]},
            {
              RowBox[{"(", GridBox[{
                    {"122", "124", "126", "128", "130", "132"},
                    {"128", "130", "132", "134", "136", "138"},
                    {"134", "136", "138", "140", "142", "144"},
                    {"140", "142", "144", "146", "148", "150"},
                    {"146", "148", "150", "152", "154", "156"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"152", "154", "156", "158", "160", "162"},
                    {"158", "160", "162", "164", "166", "168"},
                    {"164", "166", "168", "170", "172", "174"},
                    {"170", "172", "174", "176", "178", "180"},
                    {"176", "178", "180", "182", "184", "186"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"182", "184", "186", "188", "190", "192"},
                    {"188", "190", "192", "194", "196", "198"},
                    {"194", "196", "198", "200", "202", "204"},
                    {"200", "202", "204", "206", "208", "210"},
                    {"206", "208", "210", "212", "214", "216"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"212", "214", "216", "218", "220", "222"},
                    {"218", "220", "222", "224", "226", "228"},
                    {"224", "226", "228", "230", "232", "234"},
                    {"230", "232", "234", "236", "238", "240"},
                    {"236", "238", "240", "242", "244", "246"}
                    }], ")"}]},
            {
              RowBox[{"(", GridBox[{
                    {"242", "244", "246", "248", "250", "252"},
                    {"248", "250", "252", "254", "256", "258"},
                    {"254", "256", "258", "260", "262", "264"},
                    {"260", "262", "264", "266", "268", "270"},
                    {"266", "268", "270", "272", "274", "276"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"272", "274", "276", "278", "280", "282"},
                    {"278", "280", "282", "284", "286", "288"},
                    {"284", "286", "288", "290", "292", "294"},
                    {"290", "292", "294", "296", "298", "300"},
                    {"296", "298", "300", "302", "304", "306"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"302", "304", "306", "308", "310", "312"},
                    {"308", "310", "312", "314", "316", "318"},
                    {"314", "316", "318", "320", "322", "324"},
                    {"320", "322", "324", "326", "328", "330"},
                    {"326", "328", "330", "332", "334", "336"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"332", "334", "336", "338", "340", "342"},
                    {"338", "340", "342", "344", "346", "348"},
                    {"344", "346", "348", "350", "352", "354"},
                    {"350", "352", "354", "356", "358", "360"},
                    {"356", "358", "360", "362", "364", "366"}
                    }], ")"}]}
            }], ")"}],
      MatrixForm[ {{{{2, 4, 6, 8, 10, 12}, {8, 10, 12, 14, 16, 18}, {14, 16, 
        18, 20, 22, 24}, {20, 22, 24, 26, 28, 30}, {26, 28, 30, 32, 34, 
        36}}, {{32, 34, 36, 38, 40, 42}, {38, 40, 42, 44, 46, 48}, {44, 46, 
        48, 50, 52, 54}, {50, 52, 54, 56, 58, 60}, {56, 58, 60, 62, 64, 
        66}}, {{62, 64, 66, 68, 70, 72}, {68, 70, 72, 74, 76, 78}, {74, 76, 
        78, 80, 82, 84}, {80, 82, 84, 86, 88, 90}, {86, 88, 90, 92, 94, 
        96}}, {{92, 94, 96, 98, 100, 102}, {98, 100, 102, 104, 106, 108}, {
        104, 106, 108, 110, 112, 114}, {110, 112, 114, 116, 118, 120}, {116, 
        118, 120, 122, 124, 126}}}, {{{122, 124, 126, 128, 130, 132}, {128, 
        130, 132, 134, 136, 138}, {134, 136, 138, 140, 142, 144}, {140, 142, 
        144, 146, 148, 150}, {146, 148, 150, 152, 154, 156}}, {{152, 154, 
        156, 158, 160, 162}, {158, 160, 162, 164, 166, 168}, {164, 166, 168, 
        170, 172, 174}, {170, 172, 174, 176, 178, 180}, {176, 178, 180, 182, 
        184, 186}}, {{182, 184, 186, 188, 190, 192}, {188, 190, 192, 194, 
        196, 198}, {194, 196, 198, 200, 202, 204}, {200, 202, 204, 206, 208, 
        210}, {206, 208, 210, 212, 214, 216}}, {{212, 214, 216, 218, 220, 
        222}, {218, 220, 222, 224, 226, 228}, {224, 226, 228, 230, 232, 
        234}, {230, 232, 234, 236, 238, 240}, {236, 238, 240, 242, 244, 
        246}}}, {{{242, 244, 246, 248, 250, 252}, {248, 250, 252, 254, 256, 
        258}, {254, 256, 258, 260, 262, 264}, {260, 262, 264, 266, 268, 
        270}, {266, 268, 270, 272, 274, 276}}, {{272, 274, 276, 278, 280, 
        282}, {278, 280, 282, 284, 286, 288}, {284, 286, 288, 290, 292, 
        294}, {290, 292, 294, 296, 298, 300}, {296, 298, 300, 302, 304, 
        306}}, {{302, 304, 306, 308, 310, 312}, {308, 310, 312, 314, 316, 
        318}, {314, 316, 318, 320, 322, 324}, {320, 322, 324, 326, 328, 
        330}, {326, 328, 330, 332, 334, 336}}, {{332, 334, 336, 338, 340, 
        342}, {338, 340, 342, 344, 346, 348}, {344, 346, 348, 350, 352, 
        354}, {350, 352, 354, 356, 358, 360}, {356, 358, 360, 362, 364, 
        366}}}}]]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Apply", "Subsection"],

Cell["\<\
Apply is most commonly used to collapse dimensions or subarrays.  Here you \
again specify the level(s) you want to skip.  Apply really replaces the \
\"head\" of the level it's applied to.\
\>", "Text"],

Cell[CellGroupData[{

Cell["Apply[Plus,arr4d,{0}] // Dimensions", "Input"],

Cell[BoxData[
    \({4, 5, 6}\)], "Output"]
}, Open  ]],

Cell["\<\
Note that {0} is the default -- don't skip anything, just Apply over the very \
first dimension.   Shorthand is Plus @@ #.\
\>", "Text"],

Cell[CellGroupData[{

Cell["Plus@@arr4d // Dimensions", "Input"],

Cell[BoxData[
    \({4, 5, 6}\)], "Output"]
}, Open  ]],

Cell["Summing the first two dimensions:", "Text"],

Cell[CellGroupData[{

Cell["\<\
Apply[Plus,arr4d,{0,1}] // MatrixForm   (* this adds up all of the 5x6 boxes \
*)\
\>", "Input"],

Cell[BoxData[
    TagBox[
      RowBox[{"(", GridBox[{
            {"1992", "2004", "2016", "2028", "2040", "2052"},
            {"2064", "2076", "2088", "2100", "2112", "2124"},
            {"2136", "2148", "2160", "2172", "2184", "2196"},
            {"2208", "2220", "2232", "2244", "2256", "2268"},
            {"2280", "2292", "2304", "2316", "2328", "2340"}
            }], ")"}],
      (MatrixForm[ #]&)]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MatrixForm[arr4d]", "Input"],

Cell[BoxData[
    InterpretationBox[
      RowBox[{"(", GridBox[{
            {
              RowBox[{"(", GridBox[{
                    {"1", "2", "3", "4", "5", "6"},
                    {"7", "8", "9", "10", "11", "12"},
                    {"13", "14", "15", "16", "17", "18"},
                    {"19", "20", "21", "22", "23", "24"},
                    {"25", "26", "27", "28", "29", "30"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"31", "32", "33", "34", "35", "36"},
                    {"37", "38", "39", "40", "41", "42"},
                    {"43", "44", "45", "46", "47", "48"},
                    {"49", "50", "51", "52", "53", "54"},
                    {"55", "56", "57", "58", "59", "60"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"61", "62", "63", "64", "65", "66"},
                    {"67", "68", "69", "70", "71", "72"},
                    {"73", "74", "75", "76", "77", "78"},
                    {"79", "80", "81", "82", "83", "84"},
                    {"85", "86", "87", "88", "89", "90"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"91", "92", "93", "94", "95", "96"},
                    {"97", "98", "99", "100", "101", "102"},
                    {"103", "104", "105", "106", "107", "108"},
                    {"109", "110", "111", "112", "113", "114"},
                    {"115", "116", "117", "118", "119", "120"}
                    }], ")"}]},
            {
              RowBox[{"(", GridBox[{
                    {"121", "122", "123", "124", "125", "126"},
                    {"127", "128", "129", "130", "131", "132"},
                    {"133", "134", "135", "136", "137", "138"},
                    {"139", "140", "141", "142", "143", "144"},
                    {"145", "146", "147", "148", "149", "150"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"151", "152", "153", "154", "155", "156"},
                    {"157", "158", "159", "160", "161", "162"},
                    {"163", "164", "165", "166", "167", "168"},
                    {"169", "170", "171", "172", "173", "174"},
                    {"175", "176", "177", "178", "179", "180"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"181", "182", "183", "184", "185", "186"},
                    {"187", "188", "189", "190", "191", "192"},
                    {"193", "194", "195", "196", "197", "198"},
                    {"199", "200", "201", "202", "203", "204"},
                    {"205", "206", "207", "208", "209", "210"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"211", "212", "213", "214", "215", "216"},
                    {"217", "218", "219", "220", "221", "222"},
                    {"223", "224", "225", "226", "227", "228"},
                    {"229", "230", "231", "232", "233", "234"},
                    {"235", "236", "237", "238", "239", "240"}
                    }], ")"}]},
            {
              RowBox[{"(", GridBox[{
                    {"241", "242", "243", "244", "245", "246"},
                    {"247", "248", "249", "250", "251", "252"},
                    {"253", "254", "255", "256", "257", "258"},
                    {"259", "260", "261", "262", "263", "264"},
                    {"265", "266", "267", "268", "269", "270"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"271", "272", "273", "274", "275", "276"},
                    {"277", "278", "279", "280", "281", "282"},
                    {"283", "284", "285", "286", "287", "288"},
                    {"289", "290", "291", "292", "293", "294"},
                    {"295", "296", "297", "298", "299", "300"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"301", "302", "303", "304", "305", "306"},
                    {"307", "308", "309", "310", "311", "312"},
                    {"313", "314", "315", "316", "317", "318"},
                    {"319", "320", "321", "322", "323", "324"},
                    {"325", "326", "327", "328", "329", "330"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"331", "332", "333", "334", "335", "336"},
                    {"337", "338", "339", "340", "341", "342"},
                    {"343", "344", "345", "346", "347", "348"},
                    {"349", "350", "351", "352", "353", "354"},
                    {"355", "356", "357", "358", "359", "360"}
                    }], ")"}]}
            }], ")"}],
      MatrixForm[ {{{{1, 2, 3, 4, 5, 6}, {7, 8, 9, 10, 11, 12}, {13, 14, 15, 
        16, 17, 18}, {19, 20, 21, 22, 23, 24}, {25, 26, 27, 28, 29, 30}}, {{
        31, 32, 33, 34, 35, 36}, {37, 38, 39, 40, 41, 42}, {43, 44, 45, 46, 
        47, 48}, {49, 50, 51, 52, 53, 54}, {55, 56, 57, 58, 59, 60}}, {{61, 
        62, 63, 64, 65, 66}, {67, 68, 69, 70, 71, 72}, {73, 74, 75, 76, 77, 
        78}, {79, 80, 81, 82, 83, 84}, {85, 86, 87, 88, 89, 90}}, {{91, 92, 
        93, 94, 95, 96}, {97, 98, 99, 100, 101, 102}, {103, 104, 105, 106, 
        107, 108}, {109, 110, 111, 112, 113, 114}, {115, 116, 117, 118, 119, 
        120}}}, {{{121, 122, 123, 124, 125, 126}, {127, 128, 129, 130, 131, 
        132}, {133, 134, 135, 136, 137, 138}, {139, 140, 141, 142, 143, 
        144}, {145, 146, 147, 148, 149, 150}}, {{151, 152, 153, 154, 155, 
        156}, {157, 158, 159, 160, 161, 162}, {163, 164, 165, 166, 167, 
        168}, {169, 170, 171, 172, 173, 174}, {175, 176, 177, 178, 179, 
        180}}, {{181, 182, 183, 184, 185, 186}, {187, 188, 189, 190, 191, 
        192}, {193, 194, 195, 196, 197, 198}, {199, 200, 201, 202, 203, 
        204}, {205, 206, 207, 208, 209, 210}}, {{211, 212, 213, 214, 215, 
        216}, {217, 218, 219, 220, 221, 222}, {223, 224, 225, 226, 227, 
        228}, {229, 230, 231, 232, 233, 234}, {235, 236, 237, 238, 239, 
        240}}}, {{{241, 242, 243, 244, 245, 246}, {247, 248, 249, 250, 251, 
        252}, {253, 254, 255, 256, 257, 258}, {259, 260, 261, 262, 263, 
        264}, {265, 266, 267, 268, 269, 270}}, {{271, 272, 273, 274, 275, 
        276}, {277, 278, 279, 280, 281, 282}, {283, 284, 285, 286, 287, 
        288}, {289, 290, 291, 292, 293, 294}, {295, 296, 297, 298, 299, 
        300}}, {{301, 302, 303, 304, 305, 306}, {307, 308, 309, 310, 311, 
        312}, {313, 314, 315, 316, 317, 318}, {319, 320, 321, 322, 323, 
        324}, {325, 326, 327, 328, 329, 330}}, {{331, 332, 333, 334, 335, 
        336}, {337, 338, 339, 340, 341, 342}, {343, 344, 345, 346, 347, 
        348}, {349, 350, 351, 352, 353, 354}, {355, 356, 357, 358, 359, 
        360}}}}]]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Transpose", "Subsection"],

Cell["\<\
Suppose I want to rearrange the dimensions of my array.  Currently it's \
3x4x5x6.  Let's make it 3x5x6x4 instead.  Use Transpose:\
\>", "Text"],

Cell[CellGroupData[{

Cell["Dimensions[arr4d]     (* this is where we are now *)", "Input"],

Cell[BoxData[
    \({3, 4, 5, 6}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
(* specify the permutation vector of where the original indices should end up \
*)
Dimensions[ Transpose[arr4d,{1,4,2,3}] ] \
\>", "Input"],

Cell[BoxData[
    \({3, 5, 6, 4}\)], "Output"]
}, Open  ]],

Cell["\<\
One little known fact is that the elements of the permutation vector don't \
have to be unique.  Here's a nice trick to get the main diagonal of a square \
matrix.;  Transpose[ arr2d, {1,1} ]:\
\>", "Text"],

Cell[CellGroupData[{

Cell["Transpose[ Drop[#,-1]& /@ arr4d[[1,1]], {1,1} ]", "Input"],

Cell[BoxData[
    \({1, 8, 15, 22, 29}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["arr4d[[1,1]] // MatrixForm", "Input"],

Cell[BoxData[
    TagBox[
      RowBox[{"(", GridBox[{
            {"1", "2", "3", "4", "5", "6"},
            {"7", "8", "9", "10", "11", "12"},
            {"13", "14", "15", "16", "17", "18"},
            {"19", "20", "21", "22", "23", "24"},
            {"25", "26", "27", "28", "29", "30"}
            }], ")"}],
      (MatrixForm[ #]&)]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["MapThread", "Subsection"],

Cell[TextData[{
  "The crown jewel of ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ", MapThread is like Map but allows more than just one argument in the pure \
(or otherwise) function.  It is of particular utility when you want to \
combine multiple arrays over like dimensions -- just MapThread over those \
common dimensions down to the nitty-gritty.  MapThread comes up a lot in \
concatenating arrays."
}], "Text"],

Cell[CellGroupData[{

Cell["Joining two vectors (1-D arrays) to make an nx2 array.", "Subsubsection"],

Cell[CellGroupData[{

Cell["MapThread[ List, {Range[3],10+Range[3]} ] // MatrixForm", "Input"],

Cell[BoxData[
    TagBox[
      RowBox[{"(", GridBox[{
            {"1", "11"},
            {"2", "12"},
            {"3", "13"}
            }], ")"}],
      (MatrixForm[ #]&)]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Join a 3x4x2 array to a 3x4x5 array to get a 3x4x7 array.", 
  "Subsubsection"],

Cell[CellGroupData[{

Cell["\<\
arr1=Table[Random[],{3},{4},{2}];   (* Define a couple of random arrays *)
arr2=Table[Random[],{3},{4},{5}];
Dimensions /@ {arr1,arr2}\
\>", "Input"],

Cell[BoxData[
    \({{3, 4, 2}, {3, 4, 5}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MapThread[ Join, {arr1,arr2},2] // Dimensions", "Input"],

Cell[BoxData[
    \({3, 4, 7}\)], "Output"]
}, Open  ]],

Cell["\<\

 
\
\>", "Text"],

Cell["A Few Utilities (Idioms)", "Subtitle"],

Cell["\<\
Here are a few handy one-or-two liners.  We'll use some of them, and some of \
their functionality below.\
\>", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
ReduceX:  mask / arr over specified dimensions (default first)\
\>", "Subsubsection"],

Cell[BoxData[
    \(\(\(ReduceX[arr_, mask_, true_:  1, dim_:  1] := 
      \ \ \ \  (*\ true\  == \ TRUE\ or\ 1\ *) \n\t
      Module[{p}, \n\t\t\tp = Flatten[Position[mask, true], 1]; \n\t\ \ \t
        Return[\ Map[#[\([p]\)]&, arr, {dim - 1}]\ ]; \n\ \ ]; 
    \)\  (*\ end\ Module\ *) \)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["NumGEX:  Numeric, Listable,  >=", "Subsubsection"],

Cell[BoxData[
    \(NumGEX[a_, b_] := Which[a >= b, 1, True, 0]; \n
    SetAttributes[NumGEX, Listable]; \)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
ThreshDivide0X:  Generic divide; return threshold (default 0) if denominator \
is 0.\
\>", "Subsubsection"],

Cell[BoxData[
    \(ThreshDivide0X[nm_, dn_, thr_:  0] := If[dn != 0, nm/dn, thr]; \n
    SetAttributes[ThreshDivide0X, Listable]; \)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
GradeUpX:  Lowest to greatest sort order of a vector.  Identical elements are \
returned left to right.\
\>", "Subsubsection"],

Cell[BoxData[
    \(\(GradeUpX[vec_] := \n\t
      Last\ /@\ Sort[Transpose[{vec, Range[Length[vec]]}], 
          LessEqual[First[#1], First[#2]]&]; \)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
SumVecFPX:  Summarize a vector by (scattered) field partitions\
\>", "Subsubsection"],

Cell[BoxData[
    \(\(SumVecFPX[vec_, fp_] := \n\t
      Map[Plus@@vec[\([#]\)]&, \(Flatten[Position[fp, #]]&\)\ /@\ Union[fp]]; 
    \)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["AvgVecFPX:  Average a vector by (scattered) field partitions", 
  "Subsubsection"],

Cell[BoxData[
    \(\(AvgVecFPX[vec_, fp_] := \n\t
      Map[\((Plus@@vec[\([#]\)])\)/Length[#]&, 
        \(Flatten[Position[fp, #]]&\)\ /@\ Union[fp]]; \)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MatrixFormNX:  Correct display for rank 3 arrays  ", "Subsubsection"],

Cell[BoxData[
    \(MatrixFormNX[arr_] := 
      Which[TensorRank[arr] == 3, MatrixForm[Transpose[arr, {1, 3, 2}]], 
        True, MatrixForm[arr]]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["VecSubMatX:  Index a vector by a matrix of indices", "Subsubsection"],

Cell[BoxData[
    \(\(VecSubMatX[vec_, idxmat_] := \n\t
      Partition[vec[\([Flatten[idxmat]]\)], Last[Dimensions[idxmat]]]; \)\)], 
  "Input"],

Cell["\<\
Problem for the student:  Generalize VecSubMatX so that the index array can \
be of arbitrary rank.  There are at least two ways to do it.

\
\>", "Text"],

Cell["A Problem", "Subtitle"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Problem Definition", "Section"],

Cell[TextData[{
  "Suppose we have a 4-D array of data:  (x,y,t,d), where the first three \
dimensions represent measurements at  positions (x,y) at successive time \
steps, and the last dimension consists of physical observables, such as \
temperature, pressure, etc., together with a factor expressing the \
reliability of the sensor data.  For example, at a point (x,y,t), the data \
might be {100.0,4.0,0.95} indicating a temperature of 100 K, a pressure of 4 \
mTorr, and a confidence of 95% that the sensor is functioning correctly.\nOur \
goal is to extract meaningful averages from the data as follows:\na)",
  StyleBox[
  " Construct time series of spatially-averaged temperature and pressure, \
without regard to the sensor reliability.\nb) Do the same, but average \
temporally rather than spatially to guage the steady state behavior of the \
system.\nc) Develop a mask on reliable data.\nd) Perform (a) and (b) again, \
but exclude data with \"reliability factor\" below 0.7 using the mask.\ne) \
From the time-averaged output of (d), bin the data into rough isobars and \
average temperature and pressure within the bins.\nf) Repeat (e), but get the \
spatial binned averages as a function of time.", "ItemizedText"]
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Data Generation", "Section"],

Cell["\<\
For ease of screen display, let's develop the code on an artificially small \
dataset.  Later we'll read in  a more substantial  set of data and perform \
timings.\
\>", "Text"],

Cell[BoxData[
    \(\(nx = 4; \ \ \  (*\ number\ of\ sensors\ in\ x - direction\ *) \n
    ny = 2; \ \ \  (*\ number\ of\ sensors\ in\ y - direction\ *) \nnt = 6; 
    \ \ \  (*\ number\ of\ time\ measurements\ *) \nnd = 3; 
    \ \ \ \  (*\ number\ of\ data\ words\ \((T, \ P, \ quality)\)\ *) \)\)], 
  "Input"],

Cell[BoxData[
    \(SeedRandom[1234]; \ \  (*\ seed\ random\ number\ generator\ *) \n
    data = Table[Random[Real, {1, 100}], {nx}, {ny}, {nt}, {nd}]; \)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Dimensions[data]\)], "Input"],

Cell[BoxData[
    \({4, 2, 6, 3}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(MatrixFormNX[data]\)], "Input"],

Cell[BoxData[
    \(MatrixFormNX[{{{{68.4350405875436429`, 85.5674908627984187`, 
              43.776704860256359`}, {89.8464560688059243`, 
              89.1144973569692844`, 49.0694345319398372`}, {
              55.9572863767152561`, 9.28629726329563176`, 
              2.07811791230135955`}, {31.6263027797596896`, 
              63.0504456032505267`, 23.84382563878912`}, {
              43.6360133546280692`, 76.6735185386916562`, 
              81.1078921939043517`}, {48.1800674998501143`, 
              64.6799876177918253`, 54.7267076911924199`}}, {{
              57.0076245704253725`, 21.5960220725232998`, 
              77.0076017019250169`}, {51.8107840677146658`, 
              54.2069397932315411`, 96.3512994997827654`}, {
              9.57256111438137757`, 66.243293204916247`, 
              11.4302349329751806`}, {7.50484343097683659`, 
              20.4580637574121038`, 18.1738586729764133`}, {
              55.4729485562599222`, 98.218546167681211`, 
              19.3799458451107433`}, {86.547555893216721`, 
              92.4225029530093955`, 75.3747205288920874`}}}, {{{
              75.7439324904826793`, 10.8740373545250701`, 
              12.3146107591050446`}, {28.1946530290419694`, 
              12.0639448726908682`, 56.1473296633326501`}, {
              55.3069861886796765`, 7.59863095651867315`, 
              35.0563431707658512`}, {5.33654559561799768`, 
              2.10004639544813409`, 11.2473314567359206`}, {
              26.4837820563844772`, 39.0932523907017603`, 
              90.6698114624729534`}, {4.74248802575908356`, 
              7.02571829897237254`, 21.9193937177253461`}}, {{
              36.1968629062130275`, 6.52394185807787874`, 
              87.6457724538616389`}, {35.3718378245086251`, 
              43.7743599532036409`, 31.1492213291857966`}, {
              12.9018399633789538`, 25.4978004699835558`, 
              32.4597491940985927`}, {3.95456830014382241`, 
              1.8378950906880858`, 69.3504708066509056`}, {
              77.1527630054189206`, 96.3559373436251576`, 
              66.7815519199222418`}, {65.0139252110329035`, 
              76.052716609970794`, 86.1086058868892223`}}}, {{{
              41.297769863537761`, 26.9206728203311485`, 
              85.3829051474978406`}, {82.3661178611301458`, 
              35.2720515645653831`, 6.00127910260580144`}, {
              50.1860422412848006`, 76.8421760030522627`, 
              47.6262791107037575`}, {70.6294412780971825`, 
              7.4116822880811668`, 46.6929546738664758`}, {
              35.7244391473248024`, 46.1316408081136231`, 
              74.9519330939825945`}, {43.738386373722653`, 
              34.886544056636719`, 76.7811700014627263`}}, {{
              97.7991700885636738`, 47.3824490300975043`, 
              68.1049921367144772`}, {12.767244790429828`, 
              22.7464534785928806`, 61.273843143208273`}, {
              27.807222273176726`, 85.8465719700986795`, 
              37.3635483310950533`}, {78.907725282078136`, 
              92.5351707086113428`, 80.8452928674928905`}, {
              87.1775060898102616`, 3.06554927902585871`, 
              45.9088915979075817`}, {11.2158515893957022`, 
              80.7658238017290841`, 56.3725946051593851`}}}, {{{
              11.1844524505827802`, 65.0842107812820724`, 
              6.8138907077465074`}, {13.6342082314367307`, 
              76.2979083939460611`, 88.3030407798193373`}, {
              9.01472061918284417`, 66.2517592013392331`, 
              9.19291625723158034`}, {76.5357959893895323`, 
              86.2682671405899803`, 5.97791605813095738`}, {
              81.3856939840548498`, 90.6892240192908438`, 
              49.9047188094949145`}, {27.0701907760528293`, 
              88.8505232754435247`, 10.8439311517979565`}}, {{
              62.7272127196846529`, 25.0046414970269736`, 
              43.9416316775359305`}, {99.628079562402263`, 
              81.9613889179555776`, 68.6320468918675885`}, {
              33.7571792269531556`, 35.5438687811201825`, 
              76.1474982102090702`}, {55.9978386604308653`, 
              57.4592708330070944`, 47.2408280013008319`}, {
              68.1327775910262189`, 89.7460794590916322`, 
              49.2663545757755105`}, {70.7050320119113173`, 
              81.8645104504362564`, 84.768163400960681`}}}}]\)], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Solution", "Section"],

Cell[CellGroupData[{

Cell["\<\
(a) Spatially-average temperatures and pressures as a function of time.\
\>", "Subsection"],

Cell["\<\
First drop off the reliability column by extracting just the first two \
columns (temperature and pressure).\
\>", "Text"],

Cell[BoxData[
    \(\(tempress = 
      Map[\ \n\t\t\t\t\t\t\t\t\t\t\t\t\t\t\tTake[#, 2]&, \n
        \t\t\t\t\t\t\t\t\t\t\t\t\t\t\tdata, \n
        \t\t\t\t\t\t\t\t\t\t\t\t\t\t\t{3}\ \ \ \ \ \ \ \ \ \ \ \n
        \t\t\t\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ ]; \)\)], 
  "Input"],

Cell["\<\
Then sum over the first two dimensions (x and y), and divide by the number of \
elements.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(spatavg = Apply[Plus, tempress, {0, 1}]\ /\ \((nx*ny)\); \n
    Dimensions[spatavg]\)], "Input"],

Cell[BoxData[
    \({6, 2}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(spatavg\  // \ MatrixFormNX\)], "Input"],

Cell[BoxData[
    TagBox[
      RowBox[{"(", GridBox[{
            {"56.2990082096291999`", "36.1191832845827986`"},
            {"51.7024226794337682`", "51.9296930413944046`"},
            {"31.8129797504690969`", "46.6387997312905522`"},
            {"41.3116326645617615`", "41.3901052271360558`"},
            {"59.3957404731134364`", "67.4967185007777192`"},
            {"44.6516871726176667`", "65.8185408829987483`"}
            }], ")"}],
      (MatrixForm[ #]&)]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["(b) Average temporally, without regard to data quality.", "Subsection"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Apply[Plus, tempress, {2}]\ /\ nt\ \ \ \  // MatrixFormNX\)], "Input"],

Cell[BoxData[
    InterpretationBox[
      RowBox[{"(", GridBox[{
            {
              RowBox[{"(", GridBox[{
                    {"56.2801944445504531`"},
                    {"44.6527196054958164`"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"64.7287062071328911`"},
                    {"58.8575613247956397`"}
                    }], ")"}]},
            {
              RowBox[{"(", GridBox[{
                    {"32.6347312309943138`"},
                    {"38.431966201782707`"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"13.1259383781428162`"},
                    {"41.673775220924849`"}
                    }], ")"}]},
            {
              RowBox[{"(", GridBox[{
                    {"53.9903661275162249`"},
                    {"52.612453352242392`"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"37.910794590130048`"},
                    {"55.3903363780258928`"}
                    }], ")"}]},
            {
              RowBox[{"(", GridBox[{
                    {"36.4708436751165976`"},
                    {"65.1580199620680833`"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"78.90698213531528`"},
                    {"61.9299599897729535`"}
                    }], ")"}]}
            }], ")"}],
      MatrixForm[ {{{56.280194444550453, 44.652719605495811}, {
        64.728706207132888, 58.857561324795633}}, {{32.634731230994312, 
        38.431966201782707}, {13.125938378142814, 41.673775220924846}}, {{
        53.990366127516218, 52.61245335224239}, {37.910794590130045, 
        55.390336378025893}}, {{36.470843675116598, 65.158019962068082}, {
        78.906982135315275, 61.929959989772954}}}]]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["(c) Develop a mask on unreliable data.", "Subsection"],

Cell["\<\
Such masks are usually more convenient if they are numeric (1 and 0) rather \
than boolean (True and False), since they can then be used arithmetically.\
\>", "Text"],

Cell[BoxData[
    \(\(\(thr = 70.0; 
    \)\ \  (*\ anything\ less\ than\ this\ is\ excluded\ *) \)\)], "Input"],

Cell["\<\
Recall we defined a Listable, numeric relational operator above which will \
come in handy now, NumGEX.  Extract the reliability factor, and run it \
through the function.\
\>", "Text"],

Cell[BoxData[
    \(\(\(mask = NumGEX[\ Map[#[\([3]\)]&, data, {3}], thr\ ]; \)\ \ \ \ 
    \)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Dimensions[mask]\)], "Input"],

Cell[BoxData[
    \({4, 2, 6}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(MatrixFormNX[Map[#[\([3]\)]&, data, {3}]]\ \ \ \ \ \ \ \ \  (*\ 
      here' s\ the\ realiability\ factor\ *) \)\)], "Input"],

Cell[BoxData[
    InterpretationBox[
      RowBox[{"(", GridBox[{
            {
              RowBox[{"(", GridBox[{
                    {"43.776704860256359`"},
                    {"77.0076017019250169`"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"49.0694345319398372`"},
                    {"96.3512994997827654`"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"2.07811791230135955`"},
                    {"11.4302349329751806`"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"23.84382563878912`"},
                    {"18.1738586729764133`"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"81.1078921939043517`"},
                    {"19.3799458451107433`"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"54.7267076911924199`"},
                    {"75.3747205288920874`"}
                    }], ")"}]},
            {
              RowBox[{"(", GridBox[{
                    {"12.3146107591050446`"},
                    {"87.6457724538616389`"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"56.1473296633326501`"},
                    {"31.1492213291857966`"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"35.0563431707658512`"},
                    {"32.4597491940985927`"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"11.2473314567359206`"},
                    {"69.3504708066509056`"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"90.6698114624729534`"},
                    {"66.7815519199222418`"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"21.9193937177253461`"},
                    {"86.1086058868892223`"}
                    }], ")"}]},
            {
              RowBox[{"(", GridBox[{
                    {"85.3829051474978406`"},
                    {"68.1049921367144772`"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"6.00127910260580144`"},
                    {"61.273843143208273`"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"47.6262791107037575`"},
                    {"37.3635483310950533`"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"46.6929546738664758`"},
                    {"80.8452928674928905`"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"74.9519330939825945`"},
                    {"45.9088915979075817`"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"76.7811700014627263`"},
                    {"56.3725946051593851`"}
                    }], ")"}]},
            {
              RowBox[{"(", GridBox[{
                    {"6.8138907077465074`"},
                    {"43.9416316775359305`"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"88.3030407798193373`"},
                    {"68.6320468918675885`"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"9.19291625723158034`"},
                    {"76.1474982102090702`"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"5.97791605813095738`"},
                    {"47.2408280013008319`"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"49.9047188094949145`"},
                    {"49.2663545757755105`"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"10.8439311517979565`"},
                    {"84.768163400960681`"}
                    }], ")"}]}
            }], ")"}],
      MatrixForm[ {{{43.776704860256359, 77.007601701925012}, {
        49.069434531939834, 96.351299499782755}, {2.0781179123013596, 
        11.43023493297518}, {23.84382563878912, 18.173858672976412}, {
        81.107892193904348, 19.379945845110743}, {54.726707691192416, 
        75.374720528892084}}, {{12.314610759105044, 87.645772453861639}, {
        56.147329663332648, 31.149221329185796}, {35.056343170765849, 
        32.459749194098592}, {11.247331456735921, 69.350470806650904}, {
        90.66981146247295, 66.781551919922236}, {21.919393717725345, 
        86.108605886889222}}, {{85.382905147497837, 68.104992136714472}, {
        6.0012791026058014, 61.273843143208268}, {47.626279110703756, 
        37.363548331095053}, {46.692954673866474, 80.845292867492887}, {
        74.951933093982589, 45.908891597907584}, {76.781170001462726, 
        56.372594605159385}}, {{6.8138907077465074, 43.941631677535931}, {
        88.303040779819341, 68.632046891867589}, {9.1929162572315803, 
        76.147498210209065}, {5.9779160581309574, 47.240828001300834}, {
        49.904718809494909, 49.266354575775509}, {10.843931151797955, 
        84.768163400960674}}}]]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(MatrixFormNX[mask]\)], "Input"],

Cell[BoxData[
    InterpretationBox[
      RowBox[{"(", GridBox[{
            {
              RowBox[{"(", GridBox[{
                    {"0"},
                    {"1"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"0"},
                    {"1"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"0"},
                    {"0"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"0"},
                    {"0"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"1"},
                    {"0"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"0"},
                    {"1"}
                    }], ")"}]},
            {
              RowBox[{"(", GridBox[{
                    {"0"},
                    {"1"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"0"},
                    {"0"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"0"},
                    {"0"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"0"},
                    {"0"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"1"},
                    {"0"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"0"},
                    {"1"}
                    }], ")"}]},
            {
              RowBox[{"(", GridBox[{
                    {"1"},
                    {"0"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"0"},
                    {"0"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"0"},
                    {"0"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"0"},
                    {"1"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"1"},
                    {"0"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"1"},
                    {"0"}
                    }], ")"}]},
            {
              RowBox[{"(", GridBox[{
                    {"0"},
                    {"0"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"1"},
                    {"0"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"0"},
                    {"1"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"0"},
                    {"0"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"0"},
                    {"0"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"0"},
                    {"1"}
                    }], ")"}]}
            }], ")"}],
      MatrixForm[ {{{0, 1}, {0, 1}, {0, 0}, {0, 0}, {1, 0}, {0, 1}}, {{0, 
        1}, {0, 0}, {0, 0}, {0, 0}, {1, 0}, {0, 1}}, {{1, 0}, {0, 0}, {0, 
        0}, {0, 1}, {1, 0}, {1, 0}}, {{0, 0}, {1, 0}, {0, 1}, {0, 0}, {0, 
        0}, {0, 1}}}]]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
(d) Recompute the spatial and temporal averages of (a) and (b) over reliable \
data.\
\>", "Subsection"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Dimensions\ /@\ {data, mask}\)], "Input"],

Cell[BoxData[
    \({{4, 2, 6, 3}, {4, 2, 6}}\)], "Output"]
}, Open  ]],

Cell["\<\
Note that the first three dimensions of the mask and the data match.  \
Mathematica's list structure allows us to multiply the two arrays, even \
though they differ in rank; mask will be replicated over each of the three \
values in the fourth dimensions of data.  For example:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(MatrixFormNX[data*mask]\)], "Input"],

Cell[BoxData[
    InterpretationBox[
      RowBox[{"(", GridBox[{
            {
              RowBox[{"(", GridBox[{
                    {"0", "0", "0"},
                    {"0", "0", "0"},
                    {"0", "0", "0"},
                    {"0", "0", "0"},
                    {"43.6360133546280692`", "76.6735185386916562`", 
                      "81.1078921939043517`"},
                    {"0", "0", "0"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"57.0076245704253725`", "21.5960220725232998`", 
                      "77.0076017019250169`"},
                    {"51.8107840677146658`", "54.2069397932315411`", 
                      "96.3512994997827654`"},
                    {"0", "0", "0"},
                    {"0", "0", "0"},
                    {"0", "0", "0"},
                    {"86.547555893216721`", "92.4225029530093955`", 
                      "75.3747205288920874`"}
                    }], ")"}]},
            {
              RowBox[{"(", GridBox[{
                    {"0", "0", "0"},
                    {"0", "0", "0"},
                    {"0", "0", "0"},
                    {"0", "0", "0"},
                    {"26.4837820563844772`", "39.0932523907017603`", 
                      "90.6698114624729534`"},
                    {"0", "0", "0"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"36.1968629062130275`", "6.52394185807787874`", 
                      "87.6457724538616389`"},
                    {"0", "0", "0"},
                    {"0", "0", "0"},
                    {"0", "0", "0"},
                    {"0", "0", "0"},
                    {"65.0139252110329035`", "76.052716609970794`", 
                      "86.1086058868892223`"}
                    }], ")"}]},
            {
              RowBox[{"(", GridBox[{
                    {"41.297769863537761`", "26.9206728203311485`", 
                      "85.3829051474978406`"},
                    {"0", "0", "0"},
                    {"0", "0", "0"},
                    {"0", "0", "0"},
                    {"35.7244391473248024`", "46.1316408081136231`", 
                      "74.9519330939825945`"},
                    {"43.738386373722653`", "34.886544056636719`", 
                      "76.7811700014627263`"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"0", "0", "0"},
                    {"0", "0", "0"},
                    {"0", "0", "0"},
                    {"78.907725282078136`", "92.5351707086113428`", 
                      "80.8452928674928905`"},
                    {"0", "0", "0"},
                    {"0", "0", "0"}
                    }], ")"}]},
            {
              RowBox[{"(", GridBox[{
                    {"0", "0", "0"},
                    {"13.6342082314367307`", "76.2979083939460611`", 
                      "88.3030407798193373`"},
                    {"0", "0", "0"},
                    {"0", "0", "0"},
                    {"0", "0", "0"},
                    {"0", "0", "0"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"0", "0", "0"},
                    {"0", "0", "0"},
                    {"33.7571792269531556`", "35.5438687811201825`", 
                      "76.1474982102090702`"},
                    {"0", "0", "0"},
                    {"0", "0", "0"},
                    {"70.7050320119113173`", "81.8645104504362564`", 
                      "84.768163400960681`"}
                    }], ")"}]}
            }], ")"}],
      MatrixForm[ {{{{0, 0, 0}, {0, 0, 0}, {0, 0, 0}, {0, 0, 0}, {
        43.636013354628069, 76.673518538691653, 81.107892193904348}, {0, 0, 
        0}}, {{57.007624570425371, 21.596022072523297, 77.007601701925012}, {
        51.810784067714664, 54.206939793231534, 96.351299499782755}, {0, 0, 
        0}, {0, 0, 0}, {0, 0, 0}, {86.547555893216725, 92.422502953009399, 
        75.374720528892084}}}, {{{0, 0, 0}, {0, 0, 0}, {0, 0, 0}, {0, 0, 0}, {
        26.483782056384477, 39.093252390701757, 90.66981146247295}, {0, 0, 
        0}}, {{36.196862906213028, 6.5239418580778787, 87.645772453861639}, {
        0, 0, 0}, {0, 0, 0}, {0, 0, 0}, {0, 0, 0}, {65.013925211032898, 
        76.052716609970787, 86.108605886889222}}}, {{{41.297769863537759, 
        26.920672820331149, 85.382905147497837}, {0, 0, 0}, {0, 0, 0}, {0, 0, 
        0}, {35.724439147324802, 46.131640808113623, 74.951933093982589}, {
        43.738386373722655, 34.886544056636716, 76.781170001462726}}, {{0, 0, 
        0}, {0, 0, 0}, {0, 0, 0}, {78.907725282078133, 92.535170708611332, 
        80.845292867492887}, {0, 0, 0}, {0, 0, 0}}}, {{{0, 0, 0}, {
        13.63420823143673, 76.297908393946059, 88.303040779819341}, {0, 0, 
        0}, {0, 0, 0}, {0, 0, 0}, {0, 0, 0}}, {{0, 0, 0}, {0, 0, 0}, {
        33.757179226953156, 35.543868781120182, 76.147498210209065}, {0, 0, 
        0}, {0, 0, 0}, {70.705032011911314, 81.864510450436256, 
        84.768163400960674}}}}]]], "Output"]
}, Open  ]],

Cell["\<\
Hence the problem of masking out the bad data becomes trivial: just multiply \
the mask in, and sum as before.  The normalization is the number of flags set \
in the mask summed appropriately.  To avoid possible divides-by-zero, use the \
utility, ThreshDivide0X defined above.   For variety let's retain the third \
column to verify that it's averages are all greater than the applied \
threshold.\
\>", "Text"],

Cell[CellGroupData[{

Cell["Spatial", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    \(ThreshDivide0X[\ \n
        \t\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \
\ \ Apply[Plus, mask*data, {0, 1}]\ , \ \ \ \  (*\ numerator\ *) \n
        \t\t\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \
\ Apply[Plus, mask, {0, 1}]\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \  (*\ 
          denominator\ *) \n
        \t\t\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ ]\ \ \n
      \t\t\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ 
       // \ MatrixFormNX\)], "Input"],

Cell[BoxData[
    TagBox[
      RowBox[{"(", GridBox[{
            {"44.8340857800587233`", "18.3468789169774417`", 
              "83.3454264344281625`"},
            {"32.7224961495756971`", "65.2524240935888055`", 
              "92.3271701398010513`"},
            {"33.7571792269531556`", "35.5438687811201825`", 
              "76.1474982102090702`"},
            {"78.907725282078136`", "92.5351707086113428`", 
              "80.8452928674928905`"},
            {"35.2814115194457844`", "53.9661372458356769`", 
              "82.2432122501199636`"},
            {"66.5012248724709031`", "71.3065685175132912`", 
              "80.7581649545511837`"}
            }], ")"}],
      (MatrixForm[ #]&)]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Temporal", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    \(tavg = 
      ThreshDivide0X[\ \ Apply[Plus, data*mask, {2}]\ , 
        Apply[Plus, mask, {2}]\ ]; \nMatrixFormNX[tavg]\)], "Input"],

Cell[BoxData[
    InterpretationBox[
      RowBox[{"(", GridBox[{
            {
              RowBox[{"(", GridBox[{
                    {"43.6360133546280692`"},
                    {"65.1219881771189257`"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"76.6735185386916562`"},
                    {"56.0751549395880832`"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"81.1078921939043517`"},
                    {"82.9112072435332869`"}
                    }], ")"}]},
            {
              RowBox[{"(", GridBox[{
                    {"26.4837820563844772`"},
                    {"50.60539405862297`"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"39.0932523907017603`"},
                    {"41.2883292340243368`"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"90.6698114624729534`"},
                    {"86.8771891703754306`"}
                    }], ")"}]},
            {
              RowBox[{"(", GridBox[{
                    {"40.2535317948617343`"},
                    {"78.907725282078136`"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"35.9796192283604998`"},
                    {"92.5351707086113428`"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"79.0386694143143841`"},
                    {"80.8452928674928905`"}
                    }], ")"}]},
            {
              RowBox[{"(", GridBox[{
                    {"13.6342082314367307`"},
                    {"52.2311056194322365`"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"76.2979083939460611`"},
                    {"58.7041896157782261`"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"88.3030407798193373`"},
                    {"80.45783080558488`"}
                    }], ")"}]}
            }], ")"}],
      MatrixForm[ {{{43.636013354628069, 65.121988177118922}, {
        76.673518538691653, 56.075154939588082}, {81.107892193904348, 
        82.911207243533283}}, {{26.483782056384477, 50.605394058622963}, {
        39.093252390701757, 41.288329234024332}, {90.66981146247295, 
        86.877189170375431}}, {{40.253531794861736, 78.907725282078133}, {
        35.979619228360498, 92.535170708611332}, {79.038669414314384, 
        80.845292867492887}}, {{13.63420823143673, 52.231105619432235}, {
        76.297908393946059, 58.704189615778219}, {88.303040779819341, 
        80.457830805584877}}}]]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  "(e) ",
  StyleBox[
  "Bin time-averaged data into rough isobars and average temperature and \
pressure within the bins.", "ItemizedText"]
}], "Subsection"],

Cell["\<\
Let's pause to consider how this task can be done.  Clearly we need several \
things:  
\ti) \tDefine the isobars.
\tii)\tDetermine which isobar which pressure measurement belongs to.
\tiii) \tSum temperature and pressure readings according to the field \
partitions defined in (ii).
\tiv) \tCalculate the number of values falling in each isobaric range.
\tv)\tCompute the average.\
\>", "Text"],

Cell[CellGroupData[{

Cell["i) Define the isobars.", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    \(isobars = 20.0*Range[5]\)], "Input"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        StyleBox["20.`",
          StyleBoxAutoDelete->True,
          PrintPrecision->3], ",", "40.`", ",", "60.`", ",", "80.`", ",", 
        "100.`"}], "}"}]], "Output"]
}, Open  ]],

Cell["\<\
Note that there are really 6 intervals here:  (less than 20), [20,40) , ... , \
(greater than 100)\
\>", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
ii)  Determine the isobaric index of each pressure measurement.\
\>", "Subsubsection"],

Cell["Consider this function:", "Text"],

Cell[CellGroupData[{

Cell["Map[NumGEX[#[[2]],isobars]&,tavg,{2}] // MatrixFormNX", "Input"],

Cell[BoxData[
    InterpretationBox[
      RowBox[{"(", GridBox[{
            {
              RowBox[{"(", GridBox[{
                    {"1"},
                    {"1"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"1"},
                    {"1"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"1"},
                    {"0"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"0"},
                    {"0"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"0"},
                    {"0"}
                    }], ")"}]},
            {
              RowBox[{"(", GridBox[{
                    {"1"},
                    {"1"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"0"},
                    {"1"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"0"},
                    {"0"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"0"},
                    {"0"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"0"},
                    {"0"}
                    }], ")"}]},
            {
              RowBox[{"(", GridBox[{
                    {"1"},
                    {"1"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"0"},
                    {"1"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"0"},
                    {"1"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"0"},
                    {"1"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"0"},
                    {"0"}
                    }], ")"}]},
            {
              RowBox[{"(", GridBox[{
                    {"1"},
                    {"1"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"1"},
                    {"1"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"1"},
                    {"0"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"0"},
                    {"0"}
                    }], ")"}], 
              RowBox[{"(", GridBox[{
                    {"0"},
                    {"0"}
                    }], ")"}]}
            }], ")"}],
      MatrixForm[ {{{1, 1}, {1, 1}, {1, 0}, {0, 0}, {0, 0}}, {{1, 1}, {0, 
        1}, {0, 0}, {0, 0}, {0, 0}}, {{1, 1}, {0, 1}, {0, 1}, {0, 1}, {0, 
        0}}, {{1, 1}, {1, 1}, {1, 0}, {0, 0}, {0, 0}}}]]], "Output"]
}, Open  ]],

Cell["\<\
You can see that summing along the rows tells us to which pressure range the \
given pressure measurement belongs.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(fpp = Map[1 + Plus@@NumGEX[#[\([2]\)], isobars]&, tavg, {2}]; \n
    Dimensions[fpp]\)], "Input"],

Cell[BoxData[
    \({4, 2}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Dimensions[tavg]", "Input"],

Cell[BoxData[
    \({4, 2, 3}\)], "Output"]
}, Open  ]],

Cell["\<\
Check the quantization by listing the lower isobar bound of the calculated \
range.\
\>", "Text"],

Cell[CellGroupData[{

Cell["\<\
MatrixForm /@ { Map[#[[2]]&,tavg,{2}],
                fpp,
                VecSubMatX[Prepend[isobars,0.0],fpp] }\
\>", "Input"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          RowBox[{"(", GridBox[{
                {"76.6735185386916562`", "56.0751549395880832`"},
                {"39.0932523907017603`", "41.2883292340243368`"},
                {"35.9796192283604998`", "92.5351707086113428`"},
                {"76.2979083939460611`", "58.7041896157782261`"}
                }], ")"}],
          (MatrixForm[ #]&)], ",", 
        TagBox[
          RowBox[{"(", GridBox[{
                {"4", "3"},
                {"2", "3"},
                {"2", "5"},
                {"4", "3"}
                }], ")"}],
          (MatrixForm[ #]&)], ",", 
        TagBox[
          RowBox[{"(", GridBox[{
                {"60.`", "40.`"},
                {
                  StyleBox["20.`",
                    StyleBoxAutoDelete->True,
                    PrintPrecision->3], "40.`"},
                {
                  StyleBox["20.`",
                    StyleBoxAutoDelete->True,
                    PrintPrecision->3], "80.`"},
                {"60.`", "40.`"}
                }], ")"}],
          (MatrixForm[ #]&)]}], "}"}]], "Output"]
}, Open  ]],

Cell["\<\
Note that I've indexed the vector, isobars, with the matrix, fpp, using our \
idiom, VecSubMatX.\
\>", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
iii - v) Sum temperature and pressure readings according to the field \
partitions defined in (ii), and form the averages.\
\>", "Subsubsection"],

Cell["There are many ways to do this one.  Here's one.", "Text"],

Cell[TextData[{
  "First get the positions of all elements corresponding to each extant \
isobar.  ",
  StyleBox["Position",
    FontFamily->"Courier"],
  " is a surprisingly fast operator."
}], "Text"],

Cell[CellGroupData[{

Cell["\<\
Union[Flatten[fpp]]   (* this gives a sorted list of the unique field \
partitions *)\
\>", "Input"],

Cell[BoxData[
    \({2, 3, 4, 5}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(pos = \(Position[fpp, #]&\)\ /@\ Union[Flatten[fpp]]\ \ \ \ \ \ \  (*
      \ find\ where\ each\ is\ using\ Position\ *) \)\)], "Input"],

Cell[BoxData[
    \({{{2, 1}, {3, 1}}, {{1, 2}, {2, 2}, {4, 2}}, {{1, 1}, {4, 1}}, {{3, 2}}}
      \)], "Output"]
}, Open  ]],

Cell["\<\
Extract the corresponding data with each set of indices, sum, and average.\
\>", "Text"],

Cell[CellGroupData[{

Cell["\<\
binavg=Map[(Plus@@Extract[tavg,#])/Length[#]&,pos];
MatrixFormNX[binavg] \
\>", "Input"],

Cell[BoxData[
    TagBox[
      RowBox[{"(", GridBox[{
            {"33.3686569256231102`", "37.5364358095311301`", 
              "84.8542404383936776`"},
            {"55.9861626183913774`", "52.022557929796882`", 
              "83.4154090731645325`"},
            {"28.6351107930324033`", "76.4857134663188631`", 
              "84.7054664868618445`"},
            {"78.907725282078136`", "92.5351707086113428`", 
              "80.8452928674928905`"}
            }], ")"}],
      (MatrixForm[ #]&)]], "Output"]
}, Open  ]],

Cell["\<\
Of course it would be helpful to have some indication of which isobar was \
which here...\
\>", "Text"],

Cell[CellGroupData[{

Cell["\<\
relbars=Prepend[isobars,0][[Union[Flatten[fpp]]]];   (*relevant isobars *)
MapThread[Append,{binavg,relbars}] // MatrixFormNX   (* catenate them on with \
MapThread *)\
\>", "Input"],

Cell[BoxData[
    TagBox[
      RowBox[{"(", GridBox[{
            {"33.3686569256231102`", "37.5364358095311301`", 
              "84.8542404383936776`", 
              StyleBox["20.`",
                StyleBoxAutoDelete->True,
                PrintPrecision->3]},
            {"55.9861626183913774`", "52.022557929796882`", 
              "83.4154090731645325`", "40.`"},
            {"28.6351107930324033`", "76.4857134663188631`", 
              "84.7054664868618445`", "60.`"},
            {"78.907725282078136`", "92.5351707086113428`", 
              "80.8452928674928905`", "80.`"}
            }], ")"}],
      (MatrixForm[ #]&)]], "Output"]
}, Open  ]],

Cell["Let's collect everything together in a function", "Text"],

Cell["\<\
BinAvgX[arr3d_,bins_,relidx_]:=    (* relidx == index of relevant measurement \
*)
\tModule[{fpp,uniq,avg},
\t
\t  (* determine field partitions by quantizing the specified measurement *)
\t\tfpp=Map[1+Plus@@NumGEX[#[[relidx]],bins]&,arr3d,{2}]; 
\t\t
\t  (* unique fields*)
\t\tuniq=Union[Flatten[fpp]];
\t\t
\t  (* construct average of all quantities over each partition *)
\t\tavg=Map[(Plus@@Extract[arr3d,#])/Length[#]&,Position[fpp,#]& /@ uniq];
\t\t
\t  (* append the lower isobars, and return *)
\t\tReturn[MapThread[Append,{avg,Prepend[bins,0][[uniq]]}]];
\t\t
\t]; (* end Module *)\
\>", "Input"],

Cell[CellGroupData[{

Cell["BinAvgX[tavg,isobars,2] // MatrixFormNX", "Input"],

Cell[BoxData[
    TagBox[
      RowBox[{"(", GridBox[{
            {"33.3686569256231102`", "37.5364358095311301`", 
              "84.8542404383936776`", 
              StyleBox["20.`",
                StyleBoxAutoDelete->True,
                PrintPrecision->3]},
            {"55.9861626183913774`", "52.022557929796882`", 
              "83.4154090731645325`", "40.`"},
            {"28.6351107930324033`", "76.4857134663188631`", 
              "84.7054664868618445`", "60.`"},
            {"78.907725282078136`", "92.5351707086113428`", 
              "80.8452928674928905`", "80.`"}
            }], ")"}],
      (MatrixForm[ #]&)]], "Output"]
}, Open  ]],

Cell["Principal Drawback:  this function works only on 3D arrays. ", "Text"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[StyleBox[
"f) Repeat (e), but get the spatial binned averages as a function of time.", 
  "ItemizedText"]], "Subsection"],

Cell["\<\
How are we going to do this?  It would clearly be nice to use our existing \
utility, but now we have 4-D array instead.  The presence of time \"in the \
middle\" is a complication.  
The easiest approach, then, is to move time up to the front of the array, and \
then Map the just-developed BinAvgX over each of the time slices.\
\>", "Text"],

Cell[CellGroupData[{

Cell["\<\
result=Map[BinAvgX[#,isobars,2]&,Transpose[data,{2,3,1,4}]];
Dimensions[result]\
\>", "Input"],

Cell[BoxData[
    \({6}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Dimensions /@ result", "Input"],

Cell[BoxData[
    \({{5, 4}, {5, 4}, {4, 4}, {5, 4}, {5, 4}, {4, 4}}\)], "Output"]
}, Open  ]],

Cell["\<\
One Further Wish:  expand each subarray to include all 6 isobaric ranges.
L:eft for the student!\
\>", "Text"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Brute Force Solution of (f)", "Section"],

Cell["Solve the problem with crass looping.", "Text"],

Cell["\<\
BruteForceX[data_,bins_,relidx_]:=
\tModule[{nx,ny,nt,nd,numbins,sum,count,i,j,t,k,d},
\t\t{nx,ny,nt,nd}=Dimensions[data];  (* get extents for ease of looping *)
\t\tnumbins=Length[bins]+1;          (* number of ranges *)
\t\t
\t  (* Intialize sum and count for each time slice, bin, and quantity *)
\t\tsum=Table[0.0,{nt},{numbins},{nd+1}];  (* nd+1 => bin specification at \
end *)
\t\tcount=Table[0.0,{nt},{numbins}];
\t\t
\t  (* Calculate the sum and count *)
\t\tDo[
\t\t   Do[
\t\t\t  Do[
\t\t\t     k=1;
\t\t\t     While[(k<numbins) && (data[[i,j,t,relidx]]>=bins[[k]]),k++];
\t\t\t     count[[t,k]]++;
\t\t\t     Do[
\t\t\t        sum[[t,k,d]]+=data[[i,j,t,d]];
\t\t\t     , {d,1,nd} ]; (* end Do over data elements *)
\t\t\t  , {t,1,nt} ]; (* end Do over time slices *)
\t\t   , {j,1,ny} ]; (* end Do over y-positions *)
\t\t, {i,1,nx} ]; (* end Do over x-positions *)
\t\t
\t  (* Now form the averages *)
\t    Do[
\t       Do[
\t          If[count[[t,k]] != 0,
\t             Do[
\t                sum[[t,k,d]]/=count[[t,k]];
\t             , {d,1,nd} ]; (* end Do over data elements *)
\t          ]; (* end If *)
\t        (* Append bin specification *)
\t          If[k==1,sum[[t,k,nd+1]]=0.0;
\t          , (* else *)
\t             sum[[t,k,nd+1]]=bins[[k-1]];
\t          ]; (* end If *)
\t       , {k,1,numbins} ];  (* end Do over bins *)
\t    , {t,1,nt} ];  (* end Do over time slices *)       
\t    
\t   (* Return the result *)
\t     Return[sum];
\t     
\t]; (* end Module *)       \
\>", "Input"],

Cell[CellGroupData[{

Cell["\<\
bruteResult=BruteForceX[data,isobars,2];
Dimensions[bruteResult]\
\>", "Input"],

Cell[BoxData[
    \({6, 6, 4}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MatrixForm /@ bruteResult", "Input"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          RowBox[{"(", GridBox[{
                {"55.9703976983478579`", "8.69898960630147399`", 
                  "49.9801916064833484`", "0.`"},
                {"53.6775357178825984`", "24.5071121299604754`", 
                  "68.7773795089862627`", 
                  StyleBox["20.`",
                    StyleBoxAutoDelete->True,
                    PrintPrecision->3]},
                {"97.7991700885636738`", "47.3824490300975043`", 
                  "68.1049921367144772`", "40.`"},
                {"11.1844524505827802`", "65.0842107812820724`", 
                  "6.8138907077465074`", "60.`"},
                {"68.4350405875436429`", "85.5674908627984187`", 
                  "43.776704860256359`", "80.`"},
                {"0.`", "0.`", "0.`", "100.`"}
                }], ")"}],
          (MatrixForm[ #]&)], ",", 
        TagBox[
          RowBox[{"(", GridBox[{
                {"28.1946530290419694`", "12.0639448726908682`", 
                  "56.1473296633326501`", "0.`"},
                {"47.5666813257799869`", "29.0092525215791319`", 
                  "33.6375611229070381`", 
                  StyleBox["20.`",
                    StyleBoxAutoDelete->True,
                    PrintPrecision->3]},
                {"43.5913109461116476`", "48.9906498732175865`", 
                  "63.7502604144842788`", "40.`"},
                {"13.6342082314367307`", "76.2979083939460611`", 
                  "88.3030407798193373`", "60.`"},
                {"94.7372678156040848`", "85.537943137462431`", 
                  "58.8507407119037129`", "80.`"},
                {"0.`", "0.`", "0.`", "100.`"}
                }], ")"}],
          (MatrixForm[ #]&)], ",", 
        TagBox[
          RowBox[{"(", GridBox[{
                {"55.6321362826974663`", "8.4424641099071529`", 
                  "18.5672305415336058`", "0.`"},
                {"23.3295095951660558`", "30.5208346255518669`", 
                  "54.3036237021538337`", 
                  StyleBox["20.`",
                    StyleBoxAutoDelete->True,
                    PrintPrecision->3]},
                {"0.`", "0.`", "0.`", "40.`"},
                {"22.9244413249496714`", "69.7790761364359113`", 
                  "22.7498101003035069`", "60.`"},
                {"27.807222273176726`", "85.8465719700986795`", 
                  "37.3635483310950533`", "80.`"},
                {"0.`", "0.`", "0.`", "100.`"}
                }], ")"}],
          (MatrixForm[ #]&)], ",", 
        TagBox[
          RowBox[{"(", GridBox[{
                {"26.640185057953003`", "3.78320792473912881`", 
                  "42.4302523124177621`", "0.`"},
                {"7.50484343097683659`", "20.4580637574121038`", 
                  "18.1738586729764133`", 
                  StyleBox["20.`",
                    StyleBoxAutoDelete->True,
                    PrintPrecision->3]},
                {"55.9978386604308653`", "57.4592708330070944`", 
                  "47.2408280013008319`", "40.`"},
                {"31.6263027797596896`", "63.0504456032505267`", 
                  "23.84382563878912`", "60.`"},
                {"77.7217606357338297`", "89.4017189246006616`", 
                  "43.4116044628119279`", "80.`"},
                {"0.`", "0.`", "0.`", "100.`"}
                }], ")"}],
          (MatrixForm[ #]&)], ",", 
        TagBox[
          RowBox[{"(", GridBox[{
                {"87.1775060898102616`", "3.06554927902585871`", 
                  "45.9088915979075817`", "0.`"},
                {"26.4837820563844772`", "39.0932523907017603`", 
                  "90.6698114624729534`", 
                  StyleBox["20.`",
                    StyleBoxAutoDelete->True,
                    PrintPrecision->3]},
                {"35.7244391473248024`", "46.1316408081136231`", 
                  "74.9519330939825945`", "40.`"},
                {"43.6360133546280692`", "76.6735185386916562`", 
                  "81.1078921939043517`", "60.`"},
                {"70.5360457841899801`", "93.7524467474222156`", 
                  "46.3331427875758539`", "80.`"},
                {"0.`", "0.`", "0.`", "100.`"}
                }], ")"}],
          (MatrixForm[ #]&)], ",", 
        TagBox[
          RowBox[{"(", GridBox[{
                {"4.74248802575908356`", "7.02571829897237254`", 
                  "21.9193937177253461`", "0.`"},
                {"43.738386373722653`", "34.886544056636719`", 
                  "76.7811700014627263`", 
                  StyleBox["20.`",
                    StyleBoxAutoDelete->True,
                    PrintPrecision->3]},
                {"0.`", "0.`", "0.`", "40.`"},
                {"56.5969963554415045`", "70.3663521138813052`", 
                  "70.4176567890408211`", "60.`"},
                {"48.8846575676441474`", "85.9758401201545829`", 
                  "56.8398524217025258`", "80.`"},
                {"0.`", "0.`", "0.`", "100.`"}
                }], ")"}],
          (MatrixForm[ #]&)]}], "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["MatrixForm /@ result  (* this is what we got before *)", "Input"],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          RowBox[{"(", GridBox[{
                {"55.9703976983478579`", "8.69898960630147399`", 
                  "49.9801916064833484`", "0"},
                {"53.6775357178825984`", "24.5071121299604754`", 
                  "68.7773795089862627`", 
                  StyleBox["20.`",
                    StyleBoxAutoDelete->True,
                    PrintPrecision->3]},
                {"97.7991700885636738`", "47.3824490300975043`", 
                  "68.1049921367144772`", "40.`"},
                {"11.1844524505827802`", "65.0842107812820724`", 
                  "6.8138907077465074`", "60.`"},
                {"68.4350405875436429`", "85.5674908627984187`", 
                  "43.776704860256359`", "80.`"}
                }], ")"}],
          (MatrixForm[ #]&)], ",", 
        TagBox[
          RowBox[{"(", GridBox[{
                {"28.1946530290419694`", "12.0639448726908682`", 
                  "56.1473296633326501`", "0"},
                {"47.5666813257799869`", "29.0092525215791319`", 
                  "33.6375611229070381`", 
                  StyleBox["20.`",
                    StyleBoxAutoDelete->True,
                    PrintPrecision->3]},
                {"43.5913109461116476`", "48.9906498732175865`", 
                  "63.7502604144842788`", "40.`"},
                {"13.6342082314367307`", "76.2979083939460611`", 
                  "88.3030407798193373`", "60.`"},
                {"94.7372678156040848`", "85.537943137462431`", 
                  "58.8507407119037129`", "80.`"}
                }], ")"}],
          (MatrixForm[ #]&)], ",", 
        TagBox[
          RowBox[{"(", GridBox[{
                {"55.6321362826974663`", "8.4424641099071529`", 
                  "18.5672305415336058`", "0"},
                {"23.3295095951660558`", "30.5208346255518669`", 
                  "54.3036237021538337`", 
                  StyleBox["20.`",
                    StyleBoxAutoDelete->True,
                    PrintPrecision->3]},
                {"22.9244413249496714`", "69.7790761364359113`", 
                  "22.7498101003035069`", "60.`"},
                {"27.807222273176726`", "85.8465719700986795`", 
                  "37.3635483310950533`", "80.`"}
                }], ")"}],
          (MatrixForm[ #]&)], ",", 
        TagBox[
          RowBox[{"(", GridBox[{
                {"26.640185057953003`", "3.78320792473912881`", 
                  "42.4302523124177621`", "0"},
                {"7.50484343097683659`", "20.4580637574121038`", 
                  "18.1738586729764133`", 
                  StyleBox["20.`",
                    StyleBoxAutoDelete->True,
                    PrintPrecision->3]},
                {"55.9978386604308653`", "57.4592708330070944`", 
                  "47.2408280013008319`", "40.`"},
                {"31.6263027797596896`", "63.0504456032505267`", 
                  "23.84382563878912`", "60.`"},
                {"77.7217606357338297`", "89.4017189246006616`", 
                  "43.4116044628119279`", "80.`"}
                }], ")"}],
          (MatrixForm[ #]&)], ",", 
        TagBox[
          RowBox[{"(", GridBox[{
                {"87.1775060898102616`", "3.06554927902585871`", 
                  "45.9088915979075817`", "0"},
                {"26.4837820563844772`", "39.0932523907017603`", 
                  "90.6698114624729534`", 
                  StyleBox["20.`",
                    StyleBoxAutoDelete->True,
                    PrintPrecision->3]},
                {"35.7244391473248024`", "46.1316408081136231`", 
                  "74.9519330939825945`", "40.`"},
                {"43.6360133546280692`", "76.6735185386916562`", 
                  "81.1078921939043517`", "60.`"},
                {"70.5360457841899801`", "93.7524467474222156`", 
                  "46.3331427875758539`", "80.`"}
                }], ")"}],
          (MatrixForm[ #]&)], ",", 
        TagBox[
          RowBox[{"(", GridBox[{
                {"4.74248802575908356`", "7.02571829897237254`", 
                  "21.9193937177253461`", "0"},
                {"43.738386373722653`", "34.886544056636719`", 
                  "76.7811700014627263`", 
                  StyleBox["20.`",
                    StyleBoxAutoDelete->True,
                    PrintPrecision->3]},
                {"56.5969963554415045`", "70.3663521138813052`", 
                  "70.4176567890408211`", "60.`"},
                {"48.8846575676441474`", "85.9758401201545829`", 
                  "56.8398524217025258`", "80.`"}
                }], ")"}],
          (MatrixForm[ #]&)]}], "}"}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Speed Comparison", "Section"],

Cell["Make a larger dataset.", "Text"],

Cell[BoxData[
    \(\(nx = 25; \ \ \  (*\ number\ of\ sensors\ in\ x - direction\ *) \n
    ny = 20; \ \ \  (*\ number\ of\ sensors\ in\ y - direction\ *) \nnt = 50; 
    \ \ \  (*\ number\ of\ time\ measurements\ *) \nnd = 3; 
    \ \ \ \  (*\ number\ of\ data\ words\ \((T, \ P, \ quality)\)\ *) \)\)], 
  "Input"],

Cell[BoxData[
    \(SeedRandom[1234]; \ \  (*\ seed\ random\ number\ generator\ *) \n
    bigdata = Table[Random[Real, {1, 100}], {nx}, {ny}, {nt}, {nd}]; \)], 
  "Input"],

Cell["Time the quick way ...", "Text"],

Cell[CellGroupData[{

Cell["\<\
outarr=Map[BinAvgX[#,isobars,2]&,Transpose[bigdata,{2,3,1,4}]]; // Timing\
\>", "Input"],

Cell[BoxData[
    \({9.82999999999992724`\ Second, Null}\)], "Output"]
}, Open  ]],

Cell["Time the slow way ...", "Text"],

Cell[CellGroupData[{

Cell["brutearr=BruteForceX[bigdata,isobars,2]; // Timing", "Input"],

Cell[BoxData[
    \({20.3199999999997116`\ Second, Null}\)], "Output"]
}, Open  ]],

Cell["\<\
Not a dramatic difference, but with only minimal effort and thought we have \
gotten a factor of 2.

\
\>", "Text"],

Cell["Summary", "Subtitle"],

Cell[TextData[{
  "Idiomatic ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "\n\t- Easy to use and modify\n\t- Quicker to code and debug than \
traditional looping constructs\n\t- Faster execution (even better with the \
new packed arrays)"
}], "Exercise"],

Cell[TextData[{
  "Guidelines to Efficient ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "\n\t- Avoid looping\n\t- Avoid pattern matching in numeric applications\n\t\
- Avoid Transpose and Outer, though they are often necessary\n\t- Avoid \
situations where memory must be repeatedly allocated and deallocated\n\t- ",
  StyleBox["Think in arrays",
    FontColor->RGBColor[1, 0, 1]]
}], "Exercise"],

Cell[TextData[{
  "Sources of Idioms\n\t- Use Your Imagination, tempered by a few guidelines\n\
\t- Look at code in some of the standard packages\n\t- Experience\n\t- Sundry \
books:  Shaw and Tigg, Wagner, ...\n\t- Upcoming Book:  ",
  StyleBox["Higher-Dimensional ",
    FontVariations->{"Underline"->True}],
  StyleBox["Mathematica",
    FontSlant->"Italic",
    FontVariations->{"Underline"->True}],
  " (Reeve)"
}], "Exercise"]
}, Open  ]]
},
FrontEndVersion->"Microsoft Windows 3.0",
ScreenRectangle->{{0, 800}, {0, 572}},
ScreenStyleEnvironment->"Working",
WindowToolbars->{},
WindowSize->{742, 450},
WindowMargins->{{-1, Automatic}, {Automatic, 5}},
PrivateNotebookOptions->{"ColorPalette"->{RGBColor, -1}},
ShowCellLabel->True,
ShowCellTags->False,
RenderingOptions->{"ObjectDithering"->True,
"RasterDithering"->False},
StyleDefinitions -> Notebook[{
  
  Cell[CellGroupData[{
  Cell["Style Definitions", "Subtitle"],
  
  Cell["\<\
Modify the definitions below to change the default appearance of all cells in \
a given style.  Make modifications to any definition using commands in the \
Format menu.\
\>", "Text"],
  
  Cell[CellGroupData[{
  
  Cell["Style Environment Names", "Section"],
  
  Cell[StyleData[All, "Working"],
    PageWidth->WindowWidth,
    ScriptMinSize->9],
  
  Cell[StyleData[All, "Presentation"],
    PageWidth->WindowWidth,
    ScriptMinSize->12,
    FontSize->16],
  
  Cell[StyleData[All, "Condensed"],
    PageWidth->WindowWidth,
    CellBracketOptions->{"Margins"->{1, 1},
    "Widths"->{0, 5}},
    ScriptMinSize->8,
    FontSize->11],
  
  Cell[StyleData[All, "Printout"],
    PageWidth->PaperWidth,
    ScriptMinSize->7,
    FontSize->10,
    PrivateFontOptions->{"FontType"->"Outline"}]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell["Notebook Options", "Section"],
  
  Cell["\<\
The options defined for the style below will be used at the Notebook level.\
\>", "Text"],
  
  Cell[StyleData["Notebook"],
    PageHeaders->{{Cell[ 
            TextData[ {
              CounterBox[ "Page"]}], "PageNumber"], None, Cell[ 
            TextData[ {
              ValueBox[ "FileName"]}], "Header"]}, {Cell[ 
            TextData[ {
              ValueBox[ "FileName"]}], "Header"], None, Cell[ 
            TextData[ {
              CounterBox[ "Page"]}], "PageNumber"]}},
    PageHeaderLines->{True, True},
    PrintingOptions->{"FirstPageHeader"->False,
    "FacingPages"->True},
    CellFrameLabelMargins->6,
    StyleMenuListing->None]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell["Styles for Headings", "Section"],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Title"],
    ShowCellBracket->False,
    CellMargins->{{0, 0}, {0, 0}},
    PageBreakBelow->False,
    LineSpacing->{1, 0},
    CounterIncrements->"Title",
    CounterAssignments->{{"Section", 0}, {"Equation", 0}, {"Figure", 0}, {
        "Subtitle", 0}, {"Subsubtitle", 0}},
    FontSize->34,
    FontColor->GrayLevel[1],
    Background->RGBColor[0, 0, 1]],
  
  Cell[StyleData["Title", "Presentation"],
    CellMargins->{{0, 0}, {0, 0}},
    LineSpacing->{1, 0},
    FontSize->44],
  
  Cell[StyleData["Title", "Condensed"],
    CellMargins->{{0, 0}, {0, 0}},
    FontSize->20],
  
  Cell[StyleData["Title", "Printout"],
    CellMargins->{{0, 0}, {0, 0}},
    FontSize->24,
    FontTracking->"Plain"]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Subtitle"],
    ShowCellBracket->False,
    CellMargins->{{0, 0}, {0, 0}},
    PageBreakBelow->False,
    LineSpacing->{1, 3},
    ParagraphIndent->-96,
    CounterIncrements->"Subtitle",
    CounterAssignments->{{"Section", 0}, {"Equation", 0}, {"Figure", 0}, {
        "Subsubtitle", 0}},
    FontFamily->"Helvetica",
    FontSize->18,
    FontColor->GrayLevel[1],
    Background->RGBColor[1, 0, 0]],
  
  Cell[StyleData["Subtitle", "Presentation"],
    CellMargins->{{0, 0}, {0, 0}},
    LineSpacing->{1, 6},
    ParagraphIndent->-157,
    FontSize->30],
  
  Cell[StyleData["Subtitle", "Condensed"],
    CellMargins->{{0, 0}, {0, 0}},
    ParagraphIndent->-78,
    FontSize->14],
  
  Cell[StyleData["Subtitle", "Printout"],
    CellMargins->{{0, 0}, {0, 0}},
    ParagraphIndent->-85,
    FontSize->16]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Subsubtitle"],
    ShowCellBracket->False,
    CellMargins->{{10, 4}, {30, 10}},
    PageBreakBelow->False,
    CounterIncrements->"Subsubtitle",
    CounterAssignments->{{"Section", 0}, {"Equation", 0}, {"Figure", 0}},
    FontFamily->"Helvetica",
    FontSize->14,
    FontSlant->"Italic"],
  
  Cell[StyleData["Subsubtitle", "Presentation"],
    CellMargins->{{8, 10}, {40, 20}},
    LineSpacing->{1, 0},
    FontSize->24],
  
  Cell[StyleData["Subsubtitle", "Condensed"],
    CellMargins->{{8, 10}, {12, 8}},
    FontSize->12],
  
  Cell[StyleData["Subsubtitle", "Printout"],
    CellMargins->{{9, 10}, {50, 10}},
    FontSize->14]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Section"],
    CellFrame->{{6, 0}, {0, 1}},
    CellDingbat->None,
    CellMargins->{{12, Inherited}, {4, 24}},
    CellGroupingRules->{"SectionGrouping", 30},
    PageBreakBelow->False,
    CellFrameMargins->6,
    CounterIncrements->"Section",
    CounterAssignments->{{"Subsection", 0}, {"Subsubsection", 0}},
    FontFamily->"Helvetica",
    FontSize->16,
    FontWeight->"Bold",
    FontColor->RGBColor[0.571389, 0.19675, 0.570504]],
  
  Cell[StyleData["Section", "Presentation"],
    CellMargins->{{10, 10}, {8, 32}},
    LineSpacing->{1, 2},
    FontSize->24,
    FontTracking->"Condensed"],
  
  Cell[StyleData["Section", "Condensed"],
    CellMargins->{{8, Inherited}, {2, 12}},
    FontSize->12],
  
  Cell[StyleData["Section", "Printout"],
    CellMargins->{{9, 0}, {2, 50}},
    FontSize->14,
    FontTracking->"Plain",
    FontColor->GrayLevel[0]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Subsection"],
    CellMargins->{{12, Inherited}, {8, 20}},
    CellGroupingRules->{"SectionGrouping", 40},
    PageBreakBelow->False,
    CounterIncrements->"Subsection",
    CounterAssignments->{{"Subsubsection", 0}},
    FontFamily->"Helvetica",
    FontSize->13,
    FontWeight->"Bold"],
  
  Cell[StyleData["Subsection", "Presentation"],
    CellMargins->{{11, 10}, {8, 32}},
    LineSpacing->{1, 0},
    FontSize->22],
  
  Cell[StyleData["Subsection", "Condensed"],
    CellMargins->{{8, Inherited}, {2, 12}},
    FontSize->12],
  
  Cell[StyleData["Subsection", "Printout"],
    CellMargins->{{9, 0}, {4, 40}},
    FontSize->12]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Subsubsection"],
    CellDingbat->"\[FilledSquare]",
    CellMargins->{{25, Inherited}, {8, 12}},
    CellGroupingRules->{"SectionGrouping", 50},
    PageBreakBelow->False,
    CounterIncrements->"Subsubsection",
    FontFamily->"Times",
    FontSize->13,
    FontWeight->"Bold"],
  
  Cell[StyleData["Subsubsection", "Presentation"],
    CellMargins->{{29, 10}, {8, 26}},
    LineSpacing->{1, 0},
    FontSize->18],
  
  Cell[StyleData["Subsubsection", "Condensed"],
    CellMargins->{{22, Inherited}, {2, 12}},
    FontSize->10],
  
  Cell[StyleData["Subsubsection", "Printout"],
    CellMargins->{{21, 0}, {4, 20}},
    FontSize->11]
  }, Closed]]
  }, Open  ]],
  
  Cell[CellGroupData[{
  
  Cell["Styles for Body Text", "Section"],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Text"],
    CellMargins->{{12, 10}, {5, 5}},
    LineSpacing->{1, 3},
    ParagraphSpacing->{0, 12},
    CounterIncrements->"Text",
    FontFamily->"Times",
    FontSize->14,
    FontColor->RGBColor[0, 0, 1]],
  
  Cell[StyleData["Text", "Presentation"],
    CellMargins->{{13, 10}, {8, 8}},
    LineSpacing->{1, 5},
    ParagraphSpacing->{0, 12}],
  
  Cell[StyleData["Text", "Condensed"],
    CellMargins->{{8, 10}, {4, 4}},
    LineSpacing->{1, 1},
    ParagraphSpacing->{0, 4}],
  
  Cell[StyleData["Text", "Printout"],
    CellMargins->{{9, 0}, {4, 4}},
    ParagraphSpacing->{0, 6}]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["SmallText"],
    CellMargins->{{12, 10}, {5, 5}},
    LineSpacing->{1, 3},
    ParagraphSpacing->{0, 6},
    CounterIncrements->"SmallText",
    FontFamily->"Helvetica",
    FontSize->9],
  
  Cell[StyleData["SmallText", "Presentation"],
    CellMargins->{{13, 10}, {8, 8}},
    LineSpacing->{1, 5},
    FontSize->12],
  
  Cell[StyleData["SmallText", "Condensed"],
    CellMargins->{{8, 10}, {2, 2}},
    LineSpacing->{1, 2},
    FontSize->9],
  
  Cell[StyleData["SmallText", "Printout"],
    CellMargins->{{9, 0}, {4, 4}},
    FontSize->7]
  }, Closed]]
  }, Open  ]],
  
  Cell[CellGroupData[{
  
  Cell["Styles for Input/Output", "Section"],
  
  Cell["\<\
The cells in this section define styles used for input and output to the \
kernel.  Be careful when modifying, renaming, or removing these styles, \
because the front end associates special meanings with these style names.\
\>", "Text"],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Input"],
    CellFrame->{{3, 0}, {0, 0}},
    CellMargins->{{52, 10}, {8, 8}},
    Evaluatable->True,
    CellGroupingRules->"InputGrouping",
    CellHorizontalScrolling->True,
    PageBreakWithin->False,
    GroupPageBreakWithin->False,
    CellLabelMargins->{{5, Inherited}, {Inherited, Inherited}},
    DefaultFormatType->DefaultInputFormatType,
    FormatType->InputForm,
    ShowStringCharacters->True,
    NumberMarks->True,
    CounterIncrements->"Input",
    FontWeight->"Bold",
    Background->RGBColor[0, 1, 1]],
  
  Cell[StyleData["Input", "Presentation"],
    CellMargins->{{62, Inherited}, {10, 10}},
    LineSpacing->{1, 0}],
  
  Cell[StyleData["Input", "Condensed"],
    CellMargins->{{40, 10}, {4, 4}}],
  
  Cell[StyleData["Input", "Printout"],
    CellMargins->{{44, 0}, {6, 6}}]
  }, Closed]],
  
  Cell[StyleData["InlineInput"],
    Evaluatable->True,
    CellGroupingRules->"InputGrouping",
    CellHorizontalScrolling->True,
    PageBreakWithin->False,
    GroupPageBreakWithin->False,
    DefaultFormatType->DefaultInputFormatType,
    AutoItalicWords->{},
    FormatType->InputForm,
    ShowStringCharacters->True,
    NumberMarks->True,
    CounterIncrements->"Input",
    FontWeight->"Bold"],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Output"],
    CellFrame->{{3, 0}, {0, 0}},
    CellMargins->{{52, 10}, {8, 8}},
    CellEditDuplicate->True,
    CellGroupingRules->"OutputGrouping",
    CellHorizontalScrolling->True,
    PageBreakWithin->False,
    GroupPageBreakWithin->False,
    GeneratedCell->True,
    CellAutoOverwrite->True,
    CellLabelMargins->{{3, Inherited}, {Inherited, Inherited}},
    DefaultFormatType->DefaultOutputFormatType,
    FormatType->InputForm,
    CounterIncrements->"Output",
    Background->RGBColor[1, 0.900008, 0.900008]],
  
  Cell[StyleData["Output", "Presentation"],
    CellMargins->{{62, Inherited}, {12, 5}},
    LineSpacing->{1, 0}],
  
  Cell[StyleData["Output", "Condensed"],
    CellMargins->{{40, Inherited}, {4, 1}}],
  
  Cell[StyleData["Output", "Printout"],
    CellMargins->{{44, 0}, {6, 2}},
    Background->GrayLevel[0.900008]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Message"],
    CellMargins->{{62, Inherited}, {Inherited, Inherited}},
    CellGroupingRules->"OutputGrouping",
    PageBreakWithin->False,
    GroupPageBreakWithin->False,
    GeneratedCell->True,
    CellAutoOverwrite->True,
    ShowCellLabel->False,
    DefaultFormatType->DefaultOutputFormatType,
    FormatType->InputForm,
    CounterIncrements->"Message",
    StyleMenuListing->None,
    FontColor->RGBColor[1, 0, 0]],
  
  Cell[StyleData["Message", "Presentation"],
    CellMargins->{{74, Inherited}, {Inherited, Inherited}},
    LineSpacing->{1, 0}],
  
  Cell[StyleData["Message", "Condensed"],
    CellMargins->{{50, Inherited}, {Inherited, Inherited}}],
  
  Cell[StyleData["Message", "Printout"],
    CellMargins->{{54, Inherited}, {Inherited, Inherited}},
    FontColor->GrayLevel[0]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Print"],
    CellMargins->{{62, Inherited}, {Inherited, Inherited}},
    CellGroupingRules->"OutputGrouping",
    CellHorizontalScrolling->True,
    PageBreakWithin->False,
    GroupPageBreakWithin->False,
    GeneratedCell->True,
    CellAutoOverwrite->True,
    ShowCellLabel->False,
    DefaultFormatType->DefaultOutputFormatType,
    FormatType->InputForm,
    CounterIncrements->"Print",
    StyleMenuListing->None],
  
  Cell[StyleData["Print", "Presentation"],
    CellMargins->{{74, Inherited}, {Inherited, Inherited}},
    LineSpacing->{1, 0}],
  
  Cell[StyleData["Print", "Condensed"],
    CellMargins->{{50, Inherited}, {Inherited, Inherited}}],
  
  Cell[StyleData["Print", "Printout"],
    CellMargins->{{54, Inherited}, {Inherited, Inherited}}]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Graphics"],
    CellMargins->{{62, Inherited}, {Inherited, Inherited}},
    CellGroupingRules->"GraphicsGrouping",
    CellHorizontalScrolling->True,
    PageBreakWithin->False,
    GeneratedCell->True,
    CellAutoOverwrite->True,
    ShowCellLabel->False,
    DefaultFormatType->DefaultOutputFormatType,
    FormatType->InputForm,
    CounterIncrements->"Graphics",
    StyleMenuListing->None],
  
  Cell[StyleData["Graphics", "Presentation"],
    CellMargins->{{74, Inherited}, {Inherited, Inherited}}],
  
  Cell[StyleData["Graphics", "Condensed"],
    CellMargins->{{52, Inherited}, {Inherited, Inherited}},
    ImageSize->{175, 175}],
  
  Cell[StyleData["Graphics", "Printout"],
    CellMargins->{{54, Inherited}, {Inherited, Inherited}},
    ImageSize->{250, 250}]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["CellLabel"],
    StyleMenuListing->None,
    FontFamily->"Helvetica",
    FontSize->11,
    FontWeight->"Bold",
    FontColor->RGBColor[0.571389, 0.19675, 0.570504]],
  
  Cell[StyleData["CellLabel", "Presentation"],
    FontSize->12],
  
  Cell[StyleData["CellLabel", "Condensed"],
    FontSize->8],
  
  Cell[StyleData["CellLabel", "Printout"],
    FontSize->8,
    FontColor->GrayLevel[0]]
  }, Closed]]
  }, Open  ]],
  
  Cell[CellGroupData[{
  
  Cell["Unique Styles", "Section"],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Author"],
    ShowCellBracket->False,
    CellMargins->{{10, 4}, {2, 10}},
    FontSize->16,
    FontSlant->"Italic"],
  
  Cell[StyleData["Author", "Presentation"],
    CellMargins->{{12, 10}, {2, 12}},
    LineSpacing->{1, 5},
    ParagraphSpacing->{0, 12},
    FontSize->20],
  
  Cell[StyleData["Author", "Condensed"],
    CellMargins->{{8, 10}, {1, 4}},
    LineSpacing->{1, 1},
    ParagraphSpacing->{0, 4},
    FontSize->12],
  
  Cell[StyleData["Author", "Printout"],
    CellMargins->{{9, 0}, {4, 12}},
    ParagraphSpacing->{0, 6},
    FontSize->14]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Copyright"],
    ShowCellBracket->False,
    CellMargins->{{10, 10}, {40, 2}},
    FontFamily->"Helvetica",
    FontSize->9],
  
  Cell[StyleData["Copyright", "Presentation"],
    CellMargins->{{12, 10}, {50, 2}},
    LineSpacing->{1, 5},
    FontSize->12],
  
  Cell[StyleData["Copyright", "Condensed"],
    CellMargins->{{8, 10}, {12, 1}},
    LineSpacing->{1, 2},
    FontSize->9],
  
  Cell[StyleData["Copyright", "Printout"],
    CellMargins->{{9, 0}, {72, 4}},
    FontSize->7]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Example"],
    CellMargins->{{12, 10}, {5, 12}},
    LineSpacing->{1, 3},
    ParagraphSpacing->{0, 12},
    CounterIncrements->"Example",
    FontFamily->"Times",
    FontWeight->"Bold"],
  
  Cell[StyleData["Example", "Presentation"],
    CellMargins->{{18, 10}, {8, 20}},
    LineSpacing->{1, 5},
    ParagraphSpacing->{0, 12}],
  
  Cell[StyleData["Example", "Condensed"],
    CellMargins->{{8, 10}, {4, 8}},
    LineSpacing->{1, 1},
    ParagraphSpacing->{0, 4}],
  
  Cell[StyleData["Example", "Printout"],
    CellMargins->{{9, 0}, {4, 10}},
    ParagraphSpacing->{0, 6}]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Definition"],
    CellFrame->{{3, 0}, {0, 0}},
    CellMargins->{{52, 10}, {8, 8}},
    PageBreakWithin->False,
    GroupPageBreakWithin->False,
    CellLabelMargins->{{23, Inherited}, {Inherited, Inherited}},
    ShowStringCharacters->True,
    CounterIncrements->"Definition",
    FontFamily->"Helvetica",
    FontWeight->"Bold",
    FontColor->GrayLevel[1],
    Background->RGBColor[0.2, 0.700008, 0.700008]],
  
  Cell[StyleData["Definition", "Presentation"],
    CellMargins->{{62, Inherited}, {5, 12}},
    LineSpacing->{1, 0}],
  
  Cell[StyleData["Definition", "Condensed"],
    CellMargins->{{40, 10}, {1, 4}}],
  
  Cell[StyleData["Definition", "Printout"],
    CellMargins->{{44, 0}, {2, 6}},
    Background->GrayLevel[0.6]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Theorem"],
    CellFrame->{{3, 0}, {0, 0}},
    CellMargins->{{52, 10}, {8, 8}},
    PageBreakWithin->False,
    GroupPageBreakWithin->False,
    CellLabelMargins->{{23, Inherited}, {Inherited, Inherited}},
    ShowStringCharacters->True,
    CounterIncrements->"Theorem",
    FontFamily->"Helvetica",
    FontWeight->"Bold",
    FontColor->GrayLevel[1],
    Background->RGBColor[0.571389, 0.19675, 0.570504]],
  
  Cell[StyleData["Theorem", "Presentation"],
    CellMargins->{{62, Inherited}, {5, 12}},
    LineSpacing->{1, 0}],
  
  Cell[StyleData["Theorem", "Condensed"],
    CellMargins->{{40, 10}, {1, 4}}],
  
  Cell[StyleData["Theorem", "Printout"],
    CellMargins->{{44, 0}, {2, 6}},
    Background->GrayLevel[0.4]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["ExerciseMain"],
    CellFrame->{{6, 0}, {0, 1}},
    CellMargins->{{12, Inherited}, {4, 20}},
    CellGroupingRules->{"SectionGrouping", 30},
    PageBreakBelow->False,
    CellFrameColor->RGBColor[0.571389, 0.19675, 0.570504],
    CounterIncrements->"Subsection",
    CounterAssignments->{{"Subsubsection", 0}},
    FontFamily->"Helvetica",
    FontSize->16,
    FontWeight->"Bold",
    FontColor->RGBColor[0.571389, 0.19675, 0.570504]],
  
  Cell[StyleData["ExerciseMain", "Presentation"],
    CellMargins->{{18, 10}, {8, 32}},
    LineSpacing->{1, 2},
    FontSize->24,
    FontTracking->"Condensed"],
  
  Cell[StyleData["ExerciseMain", "Condensed"],
    CellMargins->{{8, Inherited}, {2, 12}},
    FontSize->12],
  
  Cell[StyleData["ExerciseMain", "Printout"],
    CellMargins->{{9, 0}, {2, 50}},
    CellFrameColor->GrayLevel[0.500008],
    FontSize->14,
    FontTracking->"Plain",
    FontColor->GrayLevel[0]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Exercise"],
    CellDingbat->"\[FilledDownTriangle]",
    CellMargins->{{23, Inherited}, {4, 18}},
    CellGroupingRules->{"SectionGrouping", 50},
    PageBreakBelow->False,
    CounterIncrements->"Subsubsection",
    FontFamily->"Times",
    FontSize->13,
    FontWeight->"Bold",
    FontColor->RGBColor[0.571389, 0.19675, 0.570504]],
  
  Cell[StyleData["Exercise", "Presentation"],
    CellMargins->{{33, 10}, {8, 26}},
    LineSpacing->{1, 0},
    FontSize->18],
  
  Cell[StyleData["Exercise", "Condensed"],
    CellMargins->{{17, Inherited}, {2, 12}},
    FontSize->10],
  
  Cell[StyleData["Exercise", "Printout"],
    CellFrame->{{0, 0}, {0.5, 0}},
    CellDingbat->None,
    CellMargins->{{9, 0}, {6, 20}},
    FontSize->11,
    FontColor->GrayLevel[0]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["ExerciseText"],
    CellMargins->{{62, 10}, {5, 5}},
    LineSpacing->{1, 3},
    ParagraphSpacing->{0, 8},
    CounterIncrements->"ExcersiceText",
    FontFamily->"Times"],
  
  Cell[StyleData["ExerciseText", "Presentation"],
    CellMargins->{{74, 10}, {8, 8}},
    LineSpacing->{1, 5},
    ParagraphSpacing->{0, 12}],
  
  Cell[StyleData["ExerciseText", "Condensed"],
    CellMargins->{{52, 10}, {2, 2}},
    LineSpacing->{1, 1},
    ParagraphSpacing->{0, 4}],
  
  Cell[StyleData["ExerciseText", "Printout"],
    CellMargins->{{54, 0}, {4, 4}},
    ParagraphSpacing->{0, 6}]
  }, Closed]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell["Formulas and Programming", "Section"],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["DisplayFormula"],
    CellMargins->{{62, 10}, {2, 10}},
    CellHorizontalScrolling->True,
    DefaultFormatType->DefaultInputFormatType,
    ScriptLevel->0,
    SingleLetterItalics->True,
    UnderoverscriptBoxOptions->{LimitsPositioning->True}],
  
  Cell[StyleData["DisplayFormula", "Presentation"],
    CellMargins->{{74, 10}, {2, 10}},
    FontSize->10],
  
  Cell[StyleData["DisplayFormula", "Condensed"],
    CellMargins->{{52, 10}, {2, 10}},
    FontSize->10],
  
  Cell[StyleData["DisplayFormula", "Printout"],
    CellMargins->{{54, 10}, {2, 10}},
    FontSize->10]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["ChemicalFormula"],
    CellMargins->{{62, 10}, {2, 10}},
    DefaultFormatType->DefaultInputFormatType,
    AutoSpacing->False,
    ScriptLevel->1,
    ScriptBaselineShifts->{0.6, Automatic},
    SingleLetterItalics->False,
    ZeroWidthTimes->True],
  
  Cell[StyleData["ChemicalFormula", "Presentation"],
    CellMargins->{{74, 10}, {2, 10}},
    FontSize->10],
  
  Cell[StyleData["ChemicalFormula", "Condensed"],
    CellMargins->{{52, 10}, {2, 10}},
    FontSize->10],
  
  Cell[StyleData["ChemicalFormula", "Printout"],
    CellMargins->{{54, 10}, {2, 10}},
    FontSize->10]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Program"],
    CellMargins->{{12, 10}, {Inherited, 6}},
    FontFamily->"Courier"],
  
  Cell[StyleData["Program", "Presentation"],
    CellMargins->{{13, 30}, {Inherited, 4}},
    FontSize->9.5],
  
  Cell[StyleData["Program", "Condensed"],
    CellMargins->{{8, 10}, {Inherited, 4}},
    FontSize->9.5],
  
  Cell[StyleData["Program", "Printout"],
    CellMargins->{{9, 0}, {Inherited, 4}},
    FontSize->9.5]
  }, Closed]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell["Styles for Automatic Numbering", "Section"],
  
  Cell["\<\
The following styles are useful for numbered equations, figures, etc.  They \
automatically give the cell a FrameLabel containing a reference to a \
particular counter, and also increment that counter.\
\>", "Text"],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["NumberedEquation"],
    CellMargins->{{62, 10}, {Inherited, Inherited}},
    CellFrameLabels->{{None, Cell[ 
            TextData[ {"(", 
              CounterBox[ "NumberedEquation"], ")"}]]}, {None, None}},
    DefaultFormatType->DefaultInputFormatType,
    CounterIncrements->"NumberedEquation",
    FormatTypeAutoConvert->False],
  
  Cell[StyleData["NumberedEquation", "Presentation"],
    CellMargins->{{74, 10}, {Inherited, Inherited}}],
  
  Cell[StyleData["NumberedEquation", "Condensed"],
    CellMargins->{{52, 10}, {Inherited, Inherited}}],
  
  Cell[StyleData["NumberedEquation", "Printout"],
    CellMargins->{{54, 0}, {Inherited, Inherited}}]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["NumberedFigure"],
    CellMargins->{{62, 145}, {Inherited, Inherited}},
    CellFrameLabels->{{None, None}, {Cell[ 
            TextData[ {"Figure ", 
              CounterBox[ "NumberedFigure"]}]], None}},
    CounterIncrements->"NumberedFigure",
    ImageMargins->{{43, Inherited}, {Inherited, 0}},
    FormatTypeAutoConvert->False],
  
  Cell[StyleData["NumberedFigure", "Presentation"]],
  
  Cell[StyleData["NumberedFigure", "Condensed"]],
  
  Cell[StyleData["NumberedFigure", "Printout"]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["NumberedTable"],
    CellMargins->{{62, 145}, {Inherited, Inherited}},
    CellFrameLabels->{{None, None}, {Cell[ 
            TextData[ {"Table ", 
              CounterBox[ "NumberedTable"]}]], None}},
    TextAlignment->Center,
    CounterIncrements->"NumberedTable",
    FormatTypeAutoConvert->False],
  
  Cell[StyleData["NumberedTable", "Presentation"]],
  
  Cell[StyleData["NumberedTable", "Condensed"]],
  
  Cell[StyleData["NumberedTable", "Printout"]]
  }, Closed]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell["Styles for Headers and Footers", "Section"],
  
  Cell[StyleData["Header"],
    CellMargins->{{0, 0}, {4, 1}},
    StyleMenuListing->None,
    FontFamily->"Helvetica",
    FontSize->9,
    FontSlant->"Italic"],
  
  Cell[StyleData["Footer"],
    CellMargins->{{0, 0}, {0, 4}},
    StyleMenuListing->None,
    FontFamily->"Helvetica",
    FontSize->6],
  
  Cell[StyleData["PageNumber"],
    CellMargins->{{0, 0}, {4, 1}},
    StyleMenuListing->None,
    FontFamily->"Helvetica",
    FontSize->9,
    FontWeight->"Bold"]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell["Hyperlink Styles", "Section"],
  
  Cell["\<\
The cells below define styles useful for making hypertext ButtonBoxes.  The \
\"Hyperlink\" style is for links within the same Notebook, or between \
Notebooks.\
\>", "Text"],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["Hyperlink"],
    StyleMenuListing->None,
    ButtonStyleMenuListing->Automatic,
    FontColor->GrayLevel[1],
    Background->RGBColor[1, 0.4, 0],
    ButtonBoxOptions->{ButtonFunction:>(FrontEndExecute[ {
        FrontEnd`NotebookLocate[ #2]}]&),
    Active->True,
    ButtonFrame->"None",
    ButtonNote->ButtonData}],
  
  Cell[StyleData["Hyperlink", "Presentation"]],
  
  Cell[StyleData["Hyperlink", "Condensed"]],
  
  Cell[StyleData["Hyperlink", "Printout"],
    FontColor->GrayLevel[0],
    Background->GrayLevel[1]]
  }, Closed]],
  
  Cell["\<\
The following styles are for linking automatically to the on-line help \
system.\
\>", "Text"],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["MainBookLink"],
    StyleMenuListing->None,
    ButtonStyleMenuListing->Automatic,
    FontColor->GrayLevel[1],
    Background->RGBColor[1, 0.4, 0],
    ButtonBoxOptions->{ButtonFunction:>(FrontEndExecute[ {
        FrontEnd`HelpBrowserLookup[ "MainBook", #]}]&),
    Active->True,
    ButtonFrame->"None"}],
  
  Cell[StyleData["MainBookLink", "Presentation"]],
  
  Cell[StyleData["MainBookLink", "Condensed"]],
  
  Cell[StyleData["MainBookLink", "Printout"],
    FontColor->GrayLevel[0],
    Background->GrayLevel[1]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["AddOnsLink"],
    StyleMenuListing->None,
    ButtonStyleMenuListing->Automatic,
    FontFamily->"Courier",
    FontColor->GrayLevel[1],
    Background->RGBColor[1, 0.4, 0],
    ButtonBoxOptions->{ButtonFunction:>(FrontEndExecute[ {
        FrontEnd`HelpBrowserLookup[ "AddOns", #]}]&),
    Active->True,
    ButtonFrame->"None"}],
  
  Cell[StyleData["AddOnsLink", "Presentation"]],
  
  Cell[StyleData["AddOnsLink", "Condensed"]],
  
  Cell[StyleData["AddOnLink", "Printout"],
    FontColor->GrayLevel[0],
    Background->GrayLevel[1]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["RefGuideLink"],
    StyleMenuListing->None,
    ButtonStyleMenuListing->Automatic,
    FontFamily->"Courier",
    FontColor->GrayLevel[1],
    Background->RGBColor[1, 0.4, 0],
    ButtonBoxOptions->{ButtonFunction:>(FrontEndExecute[ {
        FrontEnd`HelpBrowserLookup[ "RefGuideLink", #]}]&),
    Active->True,
    ButtonFrame->"None"}],
  
  Cell[StyleData["RefGuideLink", "Presentation"]],
  
  Cell[StyleData["RefGuideLink", "Condensed"]],
  
  Cell[StyleData["RefGuideLink", "Printout"],
    FontColor->GrayLevel[0],
    Background->GrayLevel[1]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["GettingStartedLink"],
    StyleMenuListing->None,
    ButtonStyleMenuListing->Automatic,
    FontColor->GrayLevel[1],
    Background->RGBColor[1, 0.4, 0],
    ButtonBoxOptions->{ButtonFunction:>(FrontEndExecute[ {
        FrontEnd`HelpBrowserLookup[ "GettingStarted", #]}]&),
    Active->True,
    ButtonFrame->"None"}],
  
  Cell[StyleData["GettingStartedLink", "Presentation"]],
  
  Cell[StyleData["GettingStartedLink", "Condensed"]],
  
  Cell[StyleData["GettingStartedLink", "Printout"],
    FontColor->GrayLevel[0],
    Background->GrayLevel[1]]
  }, Closed]],
  
  Cell[CellGroupData[{
  
  Cell[StyleData["OtherInformationLink"],
    StyleMenuListing->None,
    ButtonStyleMenuListing->Automatic,
    FontColor->GrayLevel[1],
    Background->RGBColor[1, 0.4, 0],
    ButtonBoxOptions->{ButtonFunction:>(FrontEndExecute[ {
        FrontEnd`HelpBrowserLookup[ "OtherInformation", #]}]&),
    Active->True,
    ButtonFrame->"None"}],
  
  Cell[StyleData["OtherInformationLink", "Presentation"]],
  
  Cell[StyleData["OtherInformationLink", "Condensed"]],
  
  Cell[StyleData["OtherInformationLink", "Printout"],
    FontColor->GrayLevel[0],
    Background->GrayLevel[1]]
  }, Closed]]
  }, Closed]]
  }, Open  ]]
  }]
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{
Cell[1709, 49, 317, 10, 530, "Title"],
Cell[2029, 61, 28, 4, 94, "Text"],
Cell[2060, 67, 32, 0, 41, "Subtitle"],
Cell[2095, 69, 75, 3, 56, "Exercise"],
Cell[2173, 74, 195, 4, 73, "Exercise"],
Cell[2371, 80, 483, 8, 175, "Exercise"],
Cell[2857, 90, 351, 7, 107, "Exercise"],

Cell[CellGroupData[{
Cell[3233, 101, 32, 0, 56, "Section"],
Cell[3268, 103, 59, 0, 30, "Text"],
Cell[3330, 105, 69, 0, 50, "Input"],
Cell[3402, 107, 68, 0, 30, "Text"],

Cell[CellGroupData[{
Cell[3495, 111, 86, 3, 68, "Input"],
Cell[3584, 116, 49, 1, 49, "Output"]
}, Open  ]],
Cell[3648, 120, 43, 0, 30, "Text"],

Cell[CellGroupData[{
Cell[3716, 124, 239, 15, 284, "Input"],
Cell[3958, 141, 71, 1, 49, "Output"],
Cell[4032, 144, 43, 1, 49, "Output"]
}, Open  ]],
Cell[4090, 148, 43, 0, 30, "Text"],

Cell[CellGroupData[{
Cell[4158, 152, 83, 3, 68, "Input"],
Cell[4244, 157, 53, 1, 49, "Output"],
Cell[4300, 160, 43, 1, 49, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4380, 166, 71, 0, 50, "Input"],
Cell[4454, 168, 38, 1, 49, "Output"]
}, Open  ]],
Cell[4507, 172, 26, 3, 62, "Text"],
Cell[4536, 177, 42, 0, 41, "Subtitle"],
Cell[4581, 179, 25, 3, 62, "Text"],

Cell[CellGroupData[{
Cell[4631, 186, 38, 0, 46, "Subsection"],
Cell[4672, 188, 457, 11, 114, "Text"],

Cell[CellGroupData[{
Cell[5154, 203, 94, 2, 50, "Input"],
Cell[5251, 207, 49, 1, 49, "Output"]
}, Open  ]],
Cell[5315, 211, 72, 0, 30, "Text"],

Cell[CellGroupData[{
Cell[5412, 215, 131, 3, 68, "Input"],
Cell[5546, 220, 43, 1, 49, "Output"]
}, Open  ]],
Cell[5604, 224, 90, 2, 30, "Text"],

Cell[CellGroupData[{
Cell[5719, 230, 60, 3, 68, "Input"],
Cell[5782, 235, 46, 1, 49, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[5865, 241, 34, 0, 50, "Input"],
Cell[5902, 243, 6804, 119, 574, "Output"]
}, Open  ]],
Cell[12721, 365, 172, 3, 50, "Text"],

Cell[CellGroupData[{
Cell[12918, 372, 107, 3, 50, "Input"],
Cell[13028, 377, 6808, 119, 574, "Output"]
}, Open  ]],
Cell[19851, 499, 132, 3, 29, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[20020, 507, 25, 0, 46, "Subsection"],
Cell[20048, 509, 310, 7, 48, "Text"],

Cell[CellGroupData[{
Cell[20383, 520, 124, 3, 68, "Input"],
Cell[20510, 525, 6814, 119, 574, "Output"]
}, Open  ]],
Cell[27339, 647, 330, 7, 122, "Text"],
Cell[27672, 656, 113, 3, 29, "Text"],
Cell[27788, 661, 65, 0, 50, "Input"],
Cell[27856, 663, 212, 6, 29, "Text"],

Cell[CellGroupData[{
Cell[28093, 673, 118, 3, 50, "Input"],
Cell[28214, 678, 6820, 119, 574, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[35083, 803, 27, 0, 46, "Subsection"],
Cell[35113, 805, 213, 4, 48, "Text"],

Cell[CellGroupData[{
Cell[35351, 813, 52, 0, 50, "Input"],
Cell[35406, 815, 43, 1, 49, "Output"]
}, Open  ]],
Cell[35464, 819, 146, 3, 29, "Text"],

Cell[CellGroupData[{
Cell[35635, 826, 42, 0, 50, "Input"],
Cell[35680, 828, 43, 1, 49, "Output"]
}, Open  ]],
Cell[35738, 832, 49, 0, 29, "Text"],

Cell[CellGroupData[{
Cell[35812, 836, 106, 3, 50, "Input"],
Cell[35921, 841, 423, 9, 124, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[36381, 855, 34, 0, 50, "Input"],
Cell[36418, 857, 6804, 119, 574, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[43271, 982, 31, 0, 46, "Subsection"],
Cell[43305, 984, 154, 3, 29, "Text"],

Cell[CellGroupData[{
Cell[43484, 991, 69, 0, 50, "Input"],
Cell[43556, 993, 46, 1, 49, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[43639, 999, 149, 4, 68, "Input"],
Cell[43791, 1005, 46, 1, 49, "Output"]
}, Open  ]],
Cell[43852, 1009, 216, 4, 48, "Text"],

Cell[CellGroupData[{
Cell[44093, 1017, 64, 0, 50, "Input"],
Cell[44160, 1019, 52, 1, 49, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[44249, 1025, 43, 0, 50, "Input"],
Cell[44295, 1027, 354, 9, 124, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[44698, 1042, 31, 0, 46, "Subsection"],
Cell[44732, 1044, 431, 9, 67, "Text"],

Cell[CellGroupData[{
Cell[45188, 1057, 79, 0, 37, "Subsubsection"],

Cell[CellGroupData[{
Cell[45292, 1061, 72, 0, 50, "Input"],
Cell[45367, 1063, 188, 7, 92, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[45604, 1076, 85, 1, 37, "Subsubsection"],

Cell[CellGroupData[{
Cell[45714, 1081, 159, 4, 86, "Input"],
Cell[45876, 1087, 56, 1, 49, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[45969, 1093, 62, 0, 50, "Input"],
Cell[46034, 1095, 43, 1, 49, "Output"]
}, Open  ]],
Cell[46092, 1099, 27, 4, 91, "Text"],
Cell[46122, 1105, 44, 0, 41, "Subtitle"],
Cell[46169, 1107, 129, 3, 29, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[46335, 1115, 95, 2, 37, "Subsubsection"],
Cell[46433, 1119, 305, 5, 130, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[46775, 1129, 56, 0, 37, "Subsubsection"],
Cell[46834, 1131, 118, 2, 70, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[46989, 1138, 117, 3, 37, "Subsubsection"],
Cell[47109, 1143, 144, 2, 70, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[47290, 1150, 136, 3, 37, "Subsubsection"],
Cell[47429, 1155, 165, 3, 70, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[47631, 1163, 95, 2, 37, "Subsubsection"],
Cell[47729, 1167, 149, 3, 70, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[47915, 1175, 88, 1, 37, "Subsubsection"],
Cell[48006, 1178, 169, 3, 70, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[48212, 1186, 75, 0, 37, "Subsubsection"],
Cell[48290, 1188, 160, 3, 70, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[48487, 1196, 75, 0, 37, "Subsubsection"],
Cell[48565, 1198, 145, 3, 70, "Input"],
Cell[48713, 1203, 164, 5, 91, "Text"],
Cell[48880, 1210, 29, 0, 41, "Subtitle"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[48970, 1217, 37, 0, 56, "Section"],
Cell[49010, 1219, 1242, 18, 303, "Text"]
}, Closed]],

Cell[CellGroupData[{
Cell[50289, 1242, 34, 0, 36, "Section"],
Cell[50326, 1244, 187, 4, 48, "Text"],
Cell[50516, 1250, 313, 5, 110, "Input"],
Cell[50832, 1257, 165, 2, 70, "Input"],

Cell[CellGroupData[{
Cell[51022, 1263, 49, 1, 50, "Input"],
Cell[51074, 1266, 46, 1, 70, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[51157, 1272, 51, 1, 50, "Input"],
Cell[51211, 1275, 4382, 72, 70, "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[55642, 1353, 27, 0, 36, "Section"],

Cell[CellGroupData[{
Cell[55694, 1357, 101, 2, 46, "Subsection"],
Cell[55798, 1361, 132, 3, 29, "Text"],
Cell[55933, 1366, 290, 6, 130, "Input"],
Cell[56226, 1374, 113, 3, 29, "Text"],

Cell[CellGroupData[{
Cell[56364, 1381, 116, 2, 70, "Input"],
Cell[56483, 1385, 40, 1, 49, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[56560, 1391, 60, 1, 50, "Input"],
Cell[56623, 1394, 485, 10, 140, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[57157, 1410, 77, 0, 46, "Subsection"],

Cell[CellGroupData[{
Cell[57259, 1414, 90, 1, 50, "Input"],
Cell[57352, 1417, 1838, 45, 172, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[59239, 1468, 60, 0, 46, "Subsection"],
Cell[59302, 1470, 176, 3, 48, "Text"],
Cell[59481, 1475, 112, 2, 50, "Input"],
Cell[59596, 1479, 195, 4, 48, "Text"],
Cell[59794, 1485, 107, 2, 50, "Input"],

Cell[CellGroupData[{
Cell[59926, 1491, 49, 1, 50, "Input"],
Cell[59978, 1494, 43, 1, 49, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[60058, 1500, 147, 2, 50, "Input"],
Cell[60208, 1504, 5167, 120, 172, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[65412, 1629, 51, 1, 50, "Input"],
Cell[65466, 1632, 3331, 107, 172, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[68846, 1745, 114, 3, 46, "Subsection"],

Cell[CellGroupData[{
Cell[68985, 1752, 61, 1, 50, "Input"],
Cell[69049, 1755, 59, 1, 49, "Output"]
}, Open  ]],
Cell[69123, 1759, 301, 5, 48, "Text"],

Cell[CellGroupData[{
Cell[69449, 1768, 56, 1, 50, "Input"],
Cell[69508, 1771, 5044, 105, 428, "Output"]
}, Open  ]],
Cell[74567, 1879, 422, 7, 67, "Text"],

Cell[CellGroupData[{
Cell[75014, 1890, 32, 0, 37, "Subsubsection"],

Cell[CellGroupData[{
Cell[75071, 1894, 562, 9, 130, "Input"],
Cell[75636, 1905, 718, 16, 140, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[76403, 1927, 33, 0, 37, "Subsubsection"],

Cell[CellGroupData[{
Cell[76461, 1931, 153, 3, 70, "Input"],
Cell[76617, 1936, 2673, 64, 172, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[79351, 2007, 174, 5, 46, "Subsection"],
Cell[79528, 2014, 405, 9, 184, "Text"],

Cell[CellGroupData[{
Cell[79958, 2027, 47, 0, 37, "Subsubsection"],

Cell[CellGroupData[{
Cell[80030, 2031, 56, 1, 50, "Input"],
Cell[80089, 2034, 219, 6, 49, "Output"]
}, Open  ]],
Cell[80323, 2043, 122, 3, 29, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[80482, 2051, 96, 2, 37, "Subsubsection"],
Cell[80581, 2055, 39, 0, 29, "Text"],

Cell[CellGroupData[{
Cell[80645, 2059, 70, 0, 50, "Input"],
Cell[80718, 2061, 2802, 90, 172, "Output"]
}, Open  ]],
Cell[83535, 2154, 138, 3, 29, "Text"],

Cell[CellGroupData[{
Cell[83698, 2161, 117, 2, 70, "Input"],
Cell[83818, 2165, 40, 1, 49, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[83895, 2171, 33, 0, 50, "Input"],
Cell[83931, 2173, 43, 1, 49, "Output"]
}, Open  ]],
Cell[83989, 2177, 107, 3, 29, "Text"],

Cell[CellGroupData[{
Cell[84121, 2184, 139, 4, 86, "Input"],
Cell[84263, 2190, 1133, 32, 92, "Output"]
}, Open  ]],
Cell[85411, 2225, 120, 3, 29, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[85568, 2233, 155, 3, 37, "Subsubsection"],
Cell[85726, 2238, 64, 0, 29, "Text"],
Cell[85793, 2240, 202, 6, 29, "Text"],

Cell[CellGroupData[{
Cell[86020, 2250, 110, 3, 50, "Input"],
Cell[86133, 2255, 46, 1, 49, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[86216, 2261, 158, 2, 50, "Input"],
Cell[86377, 2265, 113, 2, 49, "Output"]
}, Open  ]],
Cell[86505, 2270, 98, 2, 29, "Text"],

Cell[CellGroupData[{
Cell[86628, 2276, 98, 3, 68, "Input"],
Cell[86729, 2281, 515, 12, 108, "Output"]
}, Open  ]],
Cell[87259, 2296, 113, 3, 29, "Text"],

Cell[CellGroupData[{
Cell[87397, 2303, 192, 4, 68, "Input"],
Cell[87592, 2309, 649, 15, 108, "Output"]
}, Open  ]],
Cell[88256, 2327, 63, 0, 29, "Text"],
Cell[88322, 2329, 615, 18, 320, "Input"],

Cell[CellGroupData[{
Cell[88962, 2351, 56, 0, 50, "Input"],
Cell[89021, 2353, 649, 15, 108, "Output"]
}, Open  ]],
Cell[89685, 2371, 76, 0, 29, "Text"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[89810, 2377, 135, 2, 46, "Subsection"],
Cell[89948, 2381, 353, 6, 79, "Text"],

Cell[CellGroupData[{
Cell[90326, 2391, 104, 3, 68, "Input"],
Cell[90433, 2396, 37, 1, 49, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[90507, 2402, 37, 0, 50, "Input"],
Cell[90547, 2404, 82, 1, 49, "Output"]
}, Open  ]],
Cell[90644, 2408, 120, 3, 60, "Text"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[90813, 2417, 46, 0, 56, "Section"],
Cell[90862, 2419, 53, 0, 29, "Text"],
Cell[90918, 2421, 1532, 45, 806, "Input"],

Cell[CellGroupData[{
Cell[92475, 2470, 89, 3, 68, "Input"],
Cell[92567, 2475, 43, 1, 49, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[92647, 2481, 42, 0, 50, "Input"],
Cell[92692, 2483, 5052, 108, 350, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[97781, 2596, 71, 0, 50, "Input"],
Cell[97855, 2598, 4658, 100, 294, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[102562, 2704, 35, 0, 56, "Section"],
Cell[102600, 2706, 38, 0, 29, "Text"],
Cell[102641, 2708, 316, 5, 110, "Input"],
Cell[102960, 2715, 171, 3, 70, "Input"],
Cell[103134, 2720, 38, 0, 29, "Text"],

Cell[CellGroupData[{
Cell[103197, 2724, 98, 2, 50, "Input"],
Cell[103298, 2728, 70, 1, 49, "Output"]
}, Open  ]],
Cell[103383, 2732, 37, 0, 29, "Text"],

Cell[CellGroupData[{
Cell[103445, 2736, 67, 0, 50, "Input"],
Cell[103515, 2738, 70, 1, 49, "Output"]
}, Open  ]],
Cell[103600, 2742, 125, 5, 91, "Text"],
Cell[103728, 2749, 27, 0, 41, "Subtitle"],
Cell[103758, 2751, 266, 7, 91, "Exercise"],
Cell[104027, 2760, 408, 9, 125, "Exercise"],
Cell[104438, 2771, 432, 10, 125, "Exercise"]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

