(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 3.0,
MathReader 3.0, or any compatible application. The data for the notebook 
starts with the line of stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     27426,       1062]*)
(*NotebookOutlinePosition[     28076,       1085]*)
(*  CellTagsIndexPosition[     28032,       1081]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["Style Definitions", "Subtitle"],

Cell["\<\
Modify the definitions below to change the default appearance of all cells in \
a given style.  Make modifications to any definition using commands in the \
Format menu.\
\>", "Text"],

Cell[CellGroupData[{

Cell["Style Environment Names", "Section"],

Cell[StyleData[All, "Working"],
  PageWidth->WindowWidth,
  ScriptMinSize->9],

Cell[StyleData[All, "Presentation"],
  PageWidth->WindowWidth,
  ScriptMinSize->12,
  FontSize->16],

Cell[StyleData[All, "Condensed"],
  PageWidth->WindowWidth,
  CellBracketOptions->{"Margins"->{1, 1},
  "Widths"->{0, 5}},
  ScriptMinSize->8,
  FontSize->11],

Cell[StyleData[All, "Printout"],
  PageWidth->PaperWidth,
  ScriptMinSize->7,
  FontSize->10,
  PrivateFontOptions->{"FontType"->"Outline"}]
}, Closed]],

Cell[CellGroupData[{

Cell["Notebook Options", "Section"],

Cell["\<\
The options defined for the style below will be used at the Notebook level.\
\>", "Text"],

Cell[StyleData["Notebook"],
  PageHeaders->{{Cell[ 
          TextData[ {
            CounterBox[ "Page"]}], "PageNumber"], None, Cell[ 
          TextData[ {
            ValueBox[ "FileName"]}], "Header"]}, {Cell[ 
          TextData[ {
            ValueBox[ "FileName"]}], "Header"], None, Cell[ 
          TextData[ {
            CounterBox[ "Page"]}], "PageNumber"]}},
  PageHeaderLines->{True, True},
  PrintingOptions->{"FirstPageHeader"->False,
  "FacingPages"->True},
  CellFrameLabelMargins->6,
  StyleMenuListing->None]
}, Closed]],

Cell[CellGroupData[{

Cell["Styles for Headings", "Section"],

Cell[CellGroupData[{

Cell[StyleData["Title"],
  ShowCellBracket->False,
  CellMargins->{{0, 0}, {0, 0}},
  PageBreakBelow->False,
  LineSpacing->{1, 0},
  CounterIncrements->"Title",
  CounterAssignments->{{"Section", 0}, {"Equation", 0}, {"Figure", 0}, {
      "Subtitle", 0}, {"Subsubtitle", 0}},
  FontSize->34,
  FontColor->GrayLevel[1],
  Background->RGBColor[0, 0, 1]],

Cell[StyleData["Title", "Presentation"],
  CellMargins->{{0, 0}, {0, 0}},
  LineSpacing->{1, 0},
  FontSize->44],

Cell[StyleData["Title", "Condensed"],
  CellMargins->{{0, 0}, {0, 0}},
  FontSize->20],

Cell[StyleData["Title", "Printout"],
  CellMargins->{{0, 0}, {0, 0}},
  FontSize->24,
  FontTracking->"Plain"]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["Subtitle"],
  ShowCellBracket->False,
  CellMargins->{{0, 0}, {0, 0}},
  PageBreakBelow->False,
  LineSpacing->{1, 3},
  ParagraphIndent->-96,
  CounterIncrements->"Subtitle",
  CounterAssignments->{{"Section", 0}, {"Equation", 0}, {"Figure", 0}, {
      "Subsubtitle", 0}},
  FontFamily->"Helvetica",
  FontSize->18,
  FontColor->GrayLevel[1],
  Background->RGBColor[1, 0, 0]],

Cell[StyleData["Subtitle", "Presentation"],
  CellMargins->{{0, 0}, {0, 0}},
  LineSpacing->{1, 6},
  ParagraphIndent->-157,
  FontSize->30],

Cell[StyleData["Subtitle", "Condensed"],
  CellMargins->{{0, 0}, {0, 0}},
  ParagraphIndent->-78,
  FontSize->14],

Cell[StyleData["Subtitle", "Printout"],
  CellMargins->{{0, 0}, {0, 0}},
  ParagraphIndent->-85,
  FontSize->16]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["Subsubtitle"],
  ShowCellBracket->False,
  CellMargins->{{10, 4}, {30, 10}},
  PageBreakBelow->False,
  CounterIncrements->"Subsubtitle",
  CounterAssignments->{{"Section", 0}, {"Equation", 0}, {"Figure", 0}},
  FontFamily->"Helvetica",
  FontSize->14,
  FontSlant->"Italic"],

Cell[StyleData["Subsubtitle", "Presentation"],
  CellMargins->{{8, 10}, {40, 20}},
  LineSpacing->{1, 0},
  FontSize->24],

Cell[StyleData["Subsubtitle", "Condensed"],
  CellMargins->{{8, 10}, {12, 8}},
  FontSize->12],

Cell[StyleData["Subsubtitle", "Printout"],
  CellMargins->{{9, 10}, {50, 10}},
  FontSize->14]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["Section"],
  CellFrame->{{6, 0}, {0, 1}},
  CellDingbat->None,
  CellMargins->{{12, Inherited}, {4, 24}},
  CellGroupingRules->{"SectionGrouping", 30},
  PageBreakBelow->False,
  CellFrameMargins->6,
  CounterIncrements->"Section",
  CounterAssignments->{{"Subsection", 0}, {"Subsubsection", 0}},
  FontFamily->"Helvetica",
  FontSize->16,
  FontWeight->"Bold",
  FontColor->RGBColor[0.571389, 0.19675, 0.570504]],

Cell[StyleData["Section", "Presentation"],
  CellMargins->{{10, 10}, {8, 32}},
  LineSpacing->{1, 2},
  FontSize->24,
  FontTracking->"Condensed"],

Cell[StyleData["Section", "Condensed"],
  CellMargins->{{8, Inherited}, {2, 12}},
  FontSize->12],

Cell[StyleData["Section", "Printout"],
  CellMargins->{{9, 0}, {2, 50}},
  FontSize->14,
  FontTracking->"Plain",
  FontColor->GrayLevel[0]]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["Subsection"],
  CellMargins->{{12, Inherited}, {8, 20}},
  CellGroupingRules->{"SectionGrouping", 40},
  PageBreakBelow->False,
  CounterIncrements->"Subsection",
  CounterAssignments->{{"Subsubsection", 0}},
  FontFamily->"Helvetica",
  FontSize->13,
  FontWeight->"Bold"],

Cell[StyleData["Subsection", "Presentation"],
  CellMargins->{{11, 10}, {8, 32}},
  LineSpacing->{1, 0},
  FontSize->22],

Cell[StyleData["Subsection", "Condensed"],
  CellMargins->{{8, Inherited}, {2, 12}},
  FontSize->12],

Cell[StyleData["Subsection", "Printout"],
  CellMargins->{{9, 0}, {4, 40}},
  FontSize->12]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["Subsubsection"],
  CellDingbat->"\[FilledSquare]",
  CellMargins->{{25, Inherited}, {8, 12}},
  CellGroupingRules->{"SectionGrouping", 50},
  PageBreakBelow->False,
  CounterIncrements->"Subsubsection",
  FontFamily->"Times",
  FontSize->13,
  FontWeight->"Bold"],

Cell[StyleData["Subsubsection", "Presentation"],
  CellMargins->{{29, 10}, {8, 26}},
  LineSpacing->{1, 0},
  FontSize->18],

Cell[StyleData["Subsubsection", "Condensed"],
  CellMargins->{{22, Inherited}, {2, 12}},
  FontSize->10],

Cell[StyleData["Subsubsection", "Printout"],
  CellMargins->{{21, 0}, {4, 20}},
  FontSize->11]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Styles for Body Text", "Section"],

Cell[CellGroupData[{

Cell[StyleData["Text"],
  CellMargins->{{12, 10}, {5, 5}},
  LineSpacing->{1, 3},
  ParagraphSpacing->{0, 12},
  CounterIncrements->"Text",
  FontFamily->"Times",
  FontSize->14,
  FontColor->RGBColor[0, 0, 1]],

Cell[StyleData["Text", "Presentation"],
  CellMargins->{{13, 10}, {8, 8}},
  LineSpacing->{1, 5},
  ParagraphSpacing->{0, 12}],

Cell[StyleData["Text", "Condensed"],
  CellMargins->{{8, 10}, {4, 4}},
  LineSpacing->{1, 1},
  ParagraphSpacing->{0, 4}],

Cell[StyleData["Text", "Printout"],
  CellMargins->{{9, 0}, {4, 4}},
  ParagraphSpacing->{0, 6}]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["SmallText"],
  CellMargins->{{12, 10}, {5, 5}},
  LineSpacing->{1, 3},
  ParagraphSpacing->{0, 6},
  CounterIncrements->"SmallText",
  FontFamily->"Helvetica",
  FontSize->9],

Cell[StyleData["SmallText", "Presentation"],
  CellMargins->{{13, 10}, {8, 8}},
  LineSpacing->{1, 5},
  FontSize->12],

Cell[StyleData["SmallText", "Condensed"],
  CellMargins->{{8, 10}, {2, 2}},
  LineSpacing->{1, 2},
  FontSize->9],

Cell[StyleData["SmallText", "Printout"],
  CellMargins->{{9, 0}, {4, 4}},
  FontSize->7]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Styles for Input/Output", "Section"],

Cell["\<\
The cells in this section define styles used for input and output to the \
kernel.  Be careful when modifying, renaming, or removing these styles, \
because the front end associates special meanings with these style names.\
\>", "Text"],

Cell[CellGroupData[{

Cell[StyleData["Input"],
  CellFrame->{{3, 0}, {0, 0}},
  CellMargins->{{52, 10}, {8, 8}},
  Evaluatable->True,
  CellGroupingRules->"InputGrouping",
  CellHorizontalScrolling->True,
  PageBreakWithin->False,
  GroupPageBreakWithin->False,
  CellLabelMargins->{{5, Inherited}, {Inherited, Inherited}},
  DefaultFormatType->DefaultInputFormatType,
  FormatType->InputForm,
  ShowStringCharacters->True,
  NumberMarks->True,
  CounterIncrements->"Input",
  FontWeight->"Bold",
  Background->RGBColor[0, 1, 1]],

Cell[StyleData["Input", "Presentation"],
  CellMargins->{{62, Inherited}, {10, 10}},
  LineSpacing->{1, 0}],

Cell[StyleData["Input", "Condensed"],
  CellMargins->{{40, 10}, {4, 4}}],

Cell[StyleData["Input", "Printout"],
  CellMargins->{{44, 0}, {6, 6}}]
}, Closed]],

Cell[StyleData["InlineInput"],
  Evaluatable->True,
  CellGroupingRules->"InputGrouping",
  CellHorizontalScrolling->True,
  PageBreakWithin->False,
  GroupPageBreakWithin->False,
  DefaultFormatType->DefaultInputFormatType,
  AutoItalicWords->{},
  FormatType->InputForm,
  ShowStringCharacters->True,
  NumberMarks->True,
  CounterIncrements->"Input",
  FontWeight->"Bold"],

Cell[CellGroupData[{

Cell[StyleData["Output"],
  CellFrame->{{3, 0}, {0, 0}},
  CellMargins->{{52, 10}, {8, 8}},
  CellEditDuplicate->True,
  CellGroupingRules->"OutputGrouping",
  CellHorizontalScrolling->True,
  PageBreakWithin->False,
  GroupPageBreakWithin->False,
  GeneratedCell->True,
  CellAutoOverwrite->True,
  CellLabelMargins->{{3, Inherited}, {Inherited, Inherited}},
  DefaultFormatType->DefaultOutputFormatType,
  FormatType->InputForm,
  CounterIncrements->"Output",
  Background->RGBColor[1, 0.900008, 0.900008]],

Cell[StyleData["Output", "Presentation"],
  CellMargins->{{62, Inherited}, {12, 5}},
  LineSpacing->{1, 0}],

Cell[StyleData["Output", "Condensed"],
  CellMargins->{{40, Inherited}, {4, 1}}],

Cell[StyleData["Output", "Printout"],
  CellMargins->{{44, 0}, {6, 2}},
  Background->GrayLevel[0.900008]]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["Message"],
  CellMargins->{{62, Inherited}, {Inherited, Inherited}},
  CellGroupingRules->"OutputGrouping",
  PageBreakWithin->False,
  GroupPageBreakWithin->False,
  GeneratedCell->True,
  CellAutoOverwrite->True,
  ShowCellLabel->False,
  DefaultFormatType->DefaultOutputFormatType,
  FormatType->InputForm,
  CounterIncrements->"Message",
  StyleMenuListing->None,
  FontColor->RGBColor[1, 0, 0]],

Cell[StyleData["Message", "Presentation"],
  CellMargins->{{74, Inherited}, {Inherited, Inherited}},
  LineSpacing->{1, 0}],

Cell[StyleData["Message", "Condensed"],
  CellMargins->{{50, Inherited}, {Inherited, Inherited}}],

Cell[StyleData["Message", "Printout"],
  CellMargins->{{54, Inherited}, {Inherited, Inherited}},
  FontColor->GrayLevel[0]]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["Print"],
  CellMargins->{{62, Inherited}, {Inherited, Inherited}},
  CellGroupingRules->"OutputGrouping",
  CellHorizontalScrolling->True,
  PageBreakWithin->False,
  GroupPageBreakWithin->False,
  GeneratedCell->True,
  CellAutoOverwrite->True,
  ShowCellLabel->False,
  DefaultFormatType->DefaultOutputFormatType,
  FormatType->InputForm,
  CounterIncrements->"Print",
  StyleMenuListing->None],

Cell[StyleData["Print", "Presentation"],
  CellMargins->{{74, Inherited}, {Inherited, Inherited}},
  LineSpacing->{1, 0}],

Cell[StyleData["Print", "Condensed"],
  CellMargins->{{50, Inherited}, {Inherited, Inherited}}],

Cell[StyleData["Print", "Printout"],
  CellMargins->{{54, Inherited}, {Inherited, Inherited}}]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["Graphics"],
  CellMargins->{{62, Inherited}, {Inherited, Inherited}},
  CellGroupingRules->"GraphicsGrouping",
  CellHorizontalScrolling->True,
  PageBreakWithin->False,
  GeneratedCell->True,
  CellAutoOverwrite->True,
  ShowCellLabel->False,
  DefaultFormatType->DefaultOutputFormatType,
  FormatType->InputForm,
  CounterIncrements->"Graphics",
  StyleMenuListing->None],

Cell[StyleData["Graphics", "Presentation"],
  CellMargins->{{74, Inherited}, {Inherited, Inherited}}],

Cell[StyleData["Graphics", "Condensed"],
  CellMargins->{{52, Inherited}, {Inherited, Inherited}},
  ImageSize->{175, 175}],

Cell[StyleData["Graphics", "Printout"],
  CellMargins->{{54, Inherited}, {Inherited, Inherited}},
  ImageSize->{250, 250}]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["CellLabel"],
  StyleMenuListing->None,
  FontFamily->"Helvetica",
  FontSize->11,
  FontWeight->"Bold",
  FontColor->RGBColor[0.571389, 0.19675, 0.570504]],

Cell[StyleData["CellLabel", "Presentation"],
  FontSize->12],

Cell[StyleData["CellLabel", "Condensed"],
  FontSize->8],

Cell[StyleData["CellLabel", "Printout"],
  FontSize->8,
  FontColor->GrayLevel[0]]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Unique Styles", "Section"],

Cell[CellGroupData[{

Cell[StyleData["Author"],
  ShowCellBracket->False,
  CellMargins->{{10, 4}, {2, 10}},
  FontSize->16,
  FontSlant->"Italic"],

Cell[StyleData["Author", "Presentation"],
  CellMargins->{{12, 10}, {2, 12}},
  LineSpacing->{1, 5},
  ParagraphSpacing->{0, 12},
  FontSize->20],

Cell[StyleData["Author", "Condensed"],
  CellMargins->{{8, 10}, {1, 4}},
  LineSpacing->{1, 1},
  ParagraphSpacing->{0, 4},
  FontSize->12],

Cell[StyleData["Author", "Printout"],
  CellMargins->{{9, 0}, {4, 12}},
  ParagraphSpacing->{0, 6},
  FontSize->14]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["Copyright"],
  ShowCellBracket->False,
  CellMargins->{{10, 10}, {40, 2}},
  FontFamily->"Helvetica",
  FontSize->9],

Cell[StyleData["Copyright", "Presentation"],
  CellMargins->{{12, 10}, {50, 2}},
  LineSpacing->{1, 5},
  FontSize->12],

Cell[StyleData["Copyright", "Condensed"],
  CellMargins->{{8, 10}, {12, 1}},
  LineSpacing->{1, 2},
  FontSize->9],

Cell[StyleData["Copyright", "Printout"],
  CellMargins->{{9, 0}, {72, 4}},
  FontSize->7]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["Example"],
  CellMargins->{{12, 10}, {5, 12}},
  LineSpacing->{1, 3},
  ParagraphSpacing->{0, 12},
  CounterIncrements->"Example",
  FontFamily->"Times",
  FontWeight->"Bold"],

Cell[StyleData["Example", "Presentation"],
  CellMargins->{{18, 10}, {8, 20}},
  LineSpacing->{1, 5},
  ParagraphSpacing->{0, 12}],

Cell[StyleData["Example", "Condensed"],
  CellMargins->{{8, 10}, {4, 8}},
  LineSpacing->{1, 1},
  ParagraphSpacing->{0, 4}],

Cell[StyleData["Example", "Printout"],
  CellMargins->{{9, 0}, {4, 10}},
  ParagraphSpacing->{0, 6}]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["Definition"],
  CellFrame->{{3, 0}, {0, 0}},
  CellMargins->{{52, 10}, {8, 8}},
  PageBreakWithin->False,
  GroupPageBreakWithin->False,
  CellLabelMargins->{{23, Inherited}, {Inherited, Inherited}},
  ShowStringCharacters->True,
  CounterIncrements->"Definition",
  FontFamily->"Helvetica",
  FontWeight->"Bold",
  FontColor->GrayLevel[1],
  Background->RGBColor[0.2, 0.700008, 0.700008]],

Cell[StyleData["Definition", "Presentation"],
  CellMargins->{{62, Inherited}, {5, 12}},
  LineSpacing->{1, 0}],

Cell[StyleData["Definition", "Condensed"],
  CellMargins->{{40, 10}, {1, 4}}],

Cell[StyleData["Definition", "Printout"],
  CellMargins->{{44, 0}, {2, 6}},
  Background->GrayLevel[0.6]]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["Theorem"],
  CellFrame->{{3, 0}, {0, 0}},
  CellMargins->{{52, 10}, {8, 8}},
  PageBreakWithin->False,
  GroupPageBreakWithin->False,
  CellLabelMargins->{{23, Inherited}, {Inherited, Inherited}},
  ShowStringCharacters->True,
  CounterIncrements->"Theorem",
  FontFamily->"Helvetica",
  FontWeight->"Bold",
  FontColor->GrayLevel[1],
  Background->RGBColor[0.571389, 0.19675, 0.570504]],

Cell[StyleData["Theorem", "Presentation"],
  CellMargins->{{62, Inherited}, {5, 12}},
  LineSpacing->{1, 0}],

Cell[StyleData["Theorem", "Condensed"],
  CellMargins->{{40, 10}, {1, 4}}],

Cell[StyleData["Theorem", "Printout"],
  CellMargins->{{44, 0}, {2, 6}},
  Background->GrayLevel[0.4]]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["ExerciseMain"],
  CellFrame->{{6, 0}, {0, 1}},
  CellMargins->{{12, Inherited}, {4, 20}},
  CellGroupingRules->{"SectionGrouping", 30},
  PageBreakBelow->False,
  CellFrameColor->RGBColor[0.571389, 0.19675, 0.570504],
  CounterIncrements->"Subsection",
  CounterAssignments->{{"Subsubsection", 0}},
  FontFamily->"Helvetica",
  FontSize->16,
  FontWeight->"Bold",
  FontColor->RGBColor[0.571389, 0.19675, 0.570504]],

Cell[StyleData["ExerciseMain", "Presentation"],
  CellMargins->{{18, 10}, {8, 32}},
  LineSpacing->{1, 2},
  FontSize->24,
  FontTracking->"Condensed"],

Cell[StyleData["ExerciseMain", "Condensed"],
  CellMargins->{{8, Inherited}, {2, 12}},
  FontSize->12],

Cell[StyleData["ExerciseMain", "Printout"],
  CellMargins->{{9, 0}, {2, 50}},
  CellFrameColor->GrayLevel[0.500008],
  FontSize->14,
  FontTracking->"Plain",
  FontColor->GrayLevel[0]]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["Exercise"],
  CellDingbat->"\[FilledDownTriangle]",
  CellMargins->{{23, Inherited}, {4, 18}},
  CellGroupingRules->{"SectionGrouping", 50},
  PageBreakBelow->False,
  CounterIncrements->"Subsubsection",
  FontFamily->"Times",
  FontSize->13,
  FontWeight->"Bold",
  FontColor->RGBColor[0.571389, 0.19675, 0.570504]],

Cell[StyleData["Exercise", "Presentation"],
  CellMargins->{{33, 10}, {8, 26}},
  LineSpacing->{1, 0},
  FontSize->18],

Cell[StyleData["Exercise", "Condensed"],
  CellMargins->{{17, Inherited}, {2, 12}},
  FontSize->10],

Cell[StyleData["Exercise", "Printout"],
  CellFrame->{{0, 0}, {0.5, 0}},
  CellDingbat->None,
  CellMargins->{{9, 0}, {6, 20}},
  FontSize->11,
  FontColor->GrayLevel[0]]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["ExerciseText"],
  CellMargins->{{62, 10}, {5, 5}},
  LineSpacing->{1, 3},
  ParagraphSpacing->{0, 8},
  CounterIncrements->"ExcersiceText",
  FontFamily->"Times"],

Cell[StyleData["ExerciseText", "Presentation"],
  CellMargins->{{74, 10}, {8, 8}},
  LineSpacing->{1, 5},
  ParagraphSpacing->{0, 12}],

Cell[StyleData["ExerciseText", "Condensed"],
  CellMargins->{{52, 10}, {2, 2}},
  LineSpacing->{1, 1},
  ParagraphSpacing->{0, 4}],

Cell[StyleData["ExerciseText", "Printout"],
  CellMargins->{{54, 0}, {4, 4}},
  ParagraphSpacing->{0, 6}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Formulas and Programming", "Section"],

Cell[CellGroupData[{

Cell[StyleData["DisplayFormula"],
  CellMargins->{{62, 10}, {2, 10}},
  CellHorizontalScrolling->True,
  DefaultFormatType->DefaultInputFormatType,
  ScriptLevel->0,
  SingleLetterItalics->True,
  UnderoverscriptBoxOptions->{LimitsPositioning->True}],

Cell[StyleData["DisplayFormula", "Presentation"],
  CellMargins->{{74, 10}, {2, 10}},
  FontSize->10],

Cell[StyleData["DisplayFormula", "Condensed"],
  CellMargins->{{52, 10}, {2, 10}},
  FontSize->10],

Cell[StyleData["DisplayFormula", "Printout"],
  CellMargins->{{54, 10}, {2, 10}},
  FontSize->10]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["ChemicalFormula"],
  CellMargins->{{62, 10}, {2, 10}},
  DefaultFormatType->DefaultInputFormatType,
  AutoSpacing->False,
  ScriptLevel->1,
  ScriptBaselineShifts->{0.6, Automatic},
  SingleLetterItalics->False,
  ZeroWidthTimes->True],

Cell[StyleData["ChemicalFormula", "Presentation"],
  CellMargins->{{74, 10}, {2, 10}},
  FontSize->10],

Cell[StyleData["ChemicalFormula", "Condensed"],
  CellMargins->{{52, 10}, {2, 10}},
  FontSize->10],

Cell[StyleData["ChemicalFormula", "Printout"],
  CellMargins->{{54, 10}, {2, 10}},
  FontSize->10]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["Program"],
  CellMargins->{{12, 10}, {Inherited, 6}},
  FontFamily->"Courier"],

Cell[StyleData["Program", "Presentation"],
  CellMargins->{{13, 30}, {Inherited, 4}},
  FontSize->9.5],

Cell[StyleData["Program", "Condensed"],
  CellMargins->{{8, 10}, {Inherited, 4}},
  FontSize->9.5],

Cell[StyleData["Program", "Printout"],
  CellMargins->{{9, 0}, {Inherited, 4}},
  FontSize->9.5]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Styles for Automatic Numbering", "Section"],

Cell["\<\
The following styles are useful for numbered equations, figures, etc.  They \
automatically give the cell a FrameLabel containing a reference to a \
particular counter, and also increment that counter.\
\>", "Text"],

Cell[CellGroupData[{

Cell[StyleData["NumberedEquation"],
  CellMargins->{{62, 10}, {Inherited, Inherited}},
  CellFrameLabels->{{None, Cell[ 
          TextData[ {"(", 
            CounterBox[ "NumberedEquation"], ")"}]]}, {None, None}},
  DefaultFormatType->DefaultInputFormatType,
  CounterIncrements->"NumberedEquation",
  FormatTypeAutoConvert->False],

Cell[StyleData["NumberedEquation", "Presentation"],
  CellMargins->{{74, 10}, {Inherited, Inherited}}],

Cell[StyleData["NumberedEquation", "Condensed"],
  CellMargins->{{52, 10}, {Inherited, Inherited}}],

Cell[StyleData["NumberedEquation", "Printout"],
  CellMargins->{{54, 0}, {Inherited, Inherited}}]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["NumberedFigure"],
  CellMargins->{{62, 145}, {Inherited, Inherited}},
  CellFrameLabels->{{None, None}, {Cell[ 
          TextData[ {"Figure ", 
            CounterBox[ "NumberedFigure"]}]], None}},
  CounterIncrements->"NumberedFigure",
  ImageMargins->{{43, Inherited}, {Inherited, 0}},
  FormatTypeAutoConvert->False],

Cell[StyleData["NumberedFigure", "Presentation"]],

Cell[StyleData["NumberedFigure", "Condensed"]],

Cell[StyleData["NumberedFigure", "Printout"]]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["NumberedTable"],
  CellMargins->{{62, 145}, {Inherited, Inherited}},
  CellFrameLabels->{{None, None}, {Cell[ 
          TextData[ {"Table ", 
            CounterBox[ "NumberedTable"]}]], None}},
  TextAlignment->Center,
  CounterIncrements->"NumberedTable",
  FormatTypeAutoConvert->False],

Cell[StyleData["NumberedTable", "Presentation"]],

Cell[StyleData["NumberedTable", "Condensed"]],

Cell[StyleData["NumberedTable", "Printout"]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Styles for Headers and Footers", "Section"],

Cell[StyleData["Header"],
  CellMargins->{{0, 0}, {4, 1}},
  StyleMenuListing->None,
  FontFamily->"Helvetica",
  FontSize->9,
  FontSlant->"Italic"],

Cell[StyleData["Footer"],
  CellMargins->{{0, 0}, {0, 4}},
  StyleMenuListing->None,
  FontFamily->"Helvetica",
  FontSize->6],

Cell[StyleData["PageNumber"],
  CellMargins->{{0, 0}, {4, 1}},
  StyleMenuListing->None,
  FontFamily->"Helvetica",
  FontSize->9,
  FontWeight->"Bold"]
}, Closed]],

Cell[CellGroupData[{

Cell["Hyperlink Styles", "Section"],

Cell["\<\
The cells below define styles useful for making hypertext ButtonBoxes.  The \
\"Hyperlink\" style is for links within the same Notebook, or between \
Notebooks.\
\>", "Text"],

Cell[CellGroupData[{

Cell[StyleData["Hyperlink"],
  StyleMenuListing->None,
  ButtonStyleMenuListing->Automatic,
  FontColor->GrayLevel[1],
  Background->RGBColor[1, 0.4, 0],
  ButtonBoxOptions->{ButtonFunction:>(FrontEndExecute[ {
      FrontEnd`NotebookLocate[ #2]}]&),
  Active->True,
  ButtonFrame->"None",
  ButtonNote->ButtonData}],

Cell[StyleData["Hyperlink", "Presentation"]],

Cell[StyleData["Hyperlink", "Condensed"]],

Cell[StyleData["Hyperlink", "Printout"],
  FontColor->GrayLevel[0],
  Background->GrayLevel[1]]
}, Closed]],

Cell["\<\
The following styles are for linking automatically to the on-line help \
system.\
\>", "Text"],

Cell[CellGroupData[{

Cell[StyleData["MainBookLink"],
  StyleMenuListing->None,
  ButtonStyleMenuListing->Automatic,
  FontColor->GrayLevel[1],
  Background->RGBColor[1, 0.4, 0],
  ButtonBoxOptions->{ButtonFunction:>(FrontEndExecute[ {
      FrontEnd`HelpBrowserLookup[ "MainBook", #]}]&),
  Active->True,
  ButtonFrame->"None"}],

Cell[StyleData["MainBookLink", "Presentation"]],

Cell[StyleData["MainBookLink", "Condensed"]],

Cell[StyleData["MainBookLink", "Printout"],
  FontColor->GrayLevel[0],
  Background->GrayLevel[1]]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["AddOnsLink"],
  StyleMenuListing->None,
  ButtonStyleMenuListing->Automatic,
  FontFamily->"Courier",
  FontColor->GrayLevel[1],
  Background->RGBColor[1, 0.4, 0],
  ButtonBoxOptions->{ButtonFunction:>(FrontEndExecute[ {
      FrontEnd`HelpBrowserLookup[ "AddOns", #]}]&),
  Active->True,
  ButtonFrame->"None"}],

Cell[StyleData["AddOnsLink", "Presentation"]],

Cell[StyleData["AddOnsLink", "Condensed"]],

Cell[StyleData["AddOnLink", "Printout"],
  FontColor->GrayLevel[0],
  Background->GrayLevel[1]]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["RefGuideLink"],
  StyleMenuListing->None,
  ButtonStyleMenuListing->Automatic,
  FontFamily->"Courier",
  FontColor->GrayLevel[1],
  Background->RGBColor[1, 0.4, 0],
  ButtonBoxOptions->{ButtonFunction:>(FrontEndExecute[ {
      FrontEnd`HelpBrowserLookup[ "RefGuideLink", #]}]&),
  Active->True,
  ButtonFrame->"None"}],

Cell[StyleData["RefGuideLink", "Presentation"]],

Cell[StyleData["RefGuideLink", "Condensed"]],

Cell[StyleData["RefGuideLink", "Printout"],
  FontColor->GrayLevel[0],
  Background->GrayLevel[1]]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["GettingStartedLink"],
  StyleMenuListing->None,
  ButtonStyleMenuListing->Automatic,
  FontColor->GrayLevel[1],
  Background->RGBColor[1, 0.4, 0],
  ButtonBoxOptions->{ButtonFunction:>(FrontEndExecute[ {
      FrontEnd`HelpBrowserLookup[ "GettingStarted", #]}]&),
  Active->True,
  ButtonFrame->"None"}],

Cell[StyleData["GettingStartedLink", "Presentation"]],

Cell[StyleData["GettingStartedLink", "Condensed"]],

Cell[StyleData["GettingStartedLink", "Printout"],
  FontColor->GrayLevel[0],
  Background->GrayLevel[1]]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["OtherInformationLink"],
  StyleMenuListing->None,
  ButtonStyleMenuListing->Automatic,
  FontColor->GrayLevel[1],
  Background->RGBColor[1, 0.4, 0],
  ButtonBoxOptions->{ButtonFunction:>(FrontEndExecute[ {
      FrontEnd`HelpBrowserLookup[ "OtherInformation", #]}]&),
  Active->True,
  ButtonFrame->"None"}],

Cell[StyleData["OtherInformationLink", "Presentation"]],

Cell[StyleData["OtherInformationLink", "Condensed"]],

Cell[StyleData["OtherInformationLink", "Printout"],
  FontColor->GrayLevel[0],
  Background->GrayLevel[1]]
}, Closed]]
}, Closed]]
}, Open  ]]
},
FrontEndVersion->"Microsoft Windows 3.0",
ScreenRectangle->{{0, 800}, {0, 572}},
WindowSize->{455, 506},
WindowMargins->{{Automatic, 89}, {34, Automatic}}
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1731, 51, 37, 0, 64, "Subtitle"],
Cell[1771, 53, 193, 4, 52, "Text"],

Cell[CellGroupData[{
Cell[1989, 61, 42, 0, 53, "Section"],
Cell[2034, 63, 77, 2, 70, 31, 0, "StyleData", "All", "Working"],
Cell[2114, 67, 99, 3, 70, 36, 0, "StyleData", "All", "Presentation"],
Cell[2216, 72, 158, 5, 70, 33, 0, "StyleData", "All", "Condensed"],
Cell[2377, 79, 140, 4, 70, 32, 0, "StyleData", "All", "Printout"]
}, Closed]],

Cell[CellGroupData[{
Cell[2554, 88, 35, 0, 33, "Section"],
Cell[2592, 90, 99, 2, 70, "Text"],
Cell[2694, 94, 529, 14, 70, 27, 0, "StyleData", "Notebook", "All",
  StyleMenuListing->None]
}, Closed]],

Cell[CellGroupData[{
Cell[3260, 113, 38, 0, 33, "Section"],

Cell[CellGroupData[{
Cell[3323, 117, 353, 10, 72, 24, 0, "StyleData", "Title", "All",
  CounterIncrements->"Title",
  CounterAssignments->{{"Section", 0}, {"Equation", 0}, {"Figure", 0}, {
      "Subtitle", 0}, {"Subsubtitle", 0}}],
Cell[3679, 129, 112, 3, 70, 40, 0, "StyleData", "Title", "Presentation"],
Cell[3794, 134, 86, 2, 70, 37, 0, "StyleData", "Title", "Condensed"],
Cell[3883, 138, 110, 3, 70, 36, 0, "StyleData", "Title", "Printout"]
}, Closed]],

Cell[CellGroupData[{
Cell[4030, 146, 393, 12, 58, 27, 0, "StyleData", "Subtitle", "All",
  CounterIncrements->"Subtitle",
  CounterAssignments->{{"Section", 0}, {"Equation", 0}, {"Figure", 0}, {
      "Subsubtitle", 0}}],
Cell[4426, 160, 140, 4, 70, 43, 0, "StyleData", "Subtitle", "Presentation"],
Cell[4569, 166, 113, 3, 70, 40, 0, "StyleData", "Subtitle", "Condensed"],
Cell[4685, 171, 112, 3, 70, 39, 0, "StyleData", "Subtitle", "Printout"]
}, Closed]],

Cell[CellGroupData[{
Cell[4834, 179, 291, 8, 67, 30, 0, "StyleData", "Subsubtitle", "All",
  CounterIncrements->"Subsubtitle",
  CounterAssignments->{{"Section", 0}, {"Equation", 0}, {"Figure", 0}}],
Cell[5128, 189, 121, 3, 70, 46, 0, "StyleData", "Subsubtitle", 
"Presentation"],
Cell[5252, 194, 94, 2, 70, 43, 0, "StyleData", "Subsubtitle", "Condensed"],
Cell[5349, 198, 94, 2, 70, 42, 0, "StyleData", "Subsubtitle", "Printout"]
}, Closed]],

Cell[CellGroupData[{
Cell[5480, 205, 429, 12, 51, 26, 0, "StyleData", "Section", "All",
  CounterIncrements->"Section",
  CounterAssignments->{{"Subsection", 0}, {"Subsubsection", 0}}],
Cell[5912, 219, 146, 4, 70, 42, 0, "StyleData", "Section", "Presentation"],
Cell[6061, 225, 97, 2, 70, 39, 0, "StyleData", "Section", "Condensed"],
Cell[6161, 229, 140, 4, 70, 38, 0, "StyleData", "Section", "Printout"]
}, Closed]],

Cell[CellGroupData[{
Cell[6338, 238, 289, 8, 45, 29, 0, "StyleData", "Subsection", "All",
  CounterIncrements->"Subsection",
  CounterAssignments->{{"Subsubsection", 0}}],
Cell[6630, 248, 120, 3, 70, 45, 0, "StyleData", "Subsection", 
"Presentation"],
Cell[6753, 253, 100, 2, 70, 42, 0, "StyleData", "Subsection", "Condensed"],
Cell[6856, 257, 91, 2, 70, 41, 0, "StyleData", "Subsection", "Printout"]
}, Closed]],

Cell[CellGroupData[{
Cell[6984, 264, 279, 8, 44, 32, 0, "StyleData", "Subsubsection", "All",
  CounterIncrements->"Subsubsection"],
Cell[7266, 274, 123, 3, 70, 48, 0, "StyleData", "Subsubsection", 
"Presentation"],
Cell[7392, 279, 104, 2, 70, 45, 0, "StyleData", "Subsubsection", 
"Condensed"],
Cell[7499, 283, 95, 2, 70, 44, 0, "StyleData", "Subsubsection", "Printout"]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{
Cell[7643, 291, 39, 0, 33, "Section"],

Cell[CellGroupData[{
Cell[7707, 295, 210, 7, 43, 23, 0, "StyleData", "Text", "All",
  CounterIncrements->"Text"],
Cell[7920, 304, 126, 3, 70, 39, 0, "StyleData", "Text", "Presentation"],
Cell[8049, 309, 121, 3, 70, 36, 0, "StyleData", "Text", "Condensed"],
Cell[8173, 314, 96, 2, 70, 35, 0, "StyleData", "Text", "Printout"]
}, Closed]],

Cell[CellGroupData[{
Cell[8306, 321, 190, 6, 38, 28, 0, "StyleData", "SmallText", "All",
  CounterIncrements->"SmallText"],
Cell[8499, 329, 118, 3, 70, 44, 0, "StyleData", "SmallText", "Presentation"],
Cell[8620, 334, 113, 3, 70, 41, 0, "StyleData", "SmallText", "Condensed"],
Cell[8736, 339, 88, 2, 70, 40, 0, "StyleData", "SmallText", "Printout"]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{
Cell[8873, 347, 42, 0, 33, "Section"],
Cell[8918, 349, 246, 4, 71, "Text"],

Cell[CellGroupData[{
Cell[9189, 357, 507, 15, 64, 24, 0, "StyleData", "Input", "All",
  CounterIncrements->"Input"],
Cell[9699, 374, 107, 2, 70, 40, 0, "StyleData", "Input", "Presentation"],
Cell[9809, 378, 72, 1, 70, 37, 0, "StyleData", "Input", "Condensed"],
Cell[9884, 381, 70, 1, 70, 36, 0, "StyleData", "Input", "Printout"]
}, Closed]],
Cell[9969, 385, 375, 12, 40, 30, 0, "StyleData", "InlineInput", "All",
  CounterIncrements->"Input"],

Cell[CellGroupData[{
Cell[10369, 401, 508, 14, 64, 25, 0, "StyleData", "Output", "All",
  CounterIncrements->"Output"],
Cell[10880, 417, 107, 2, 70, 41, 0, "StyleData", "Output", "Presentation"],
Cell[10990, 421, 80, 1, 70, 38, 0, "StyleData", "Output", "Condensed"],
Cell[11073, 424, 106, 2, 70, 37, 0, "StyleData", "Output", "Printout"]
}, Closed]],

Cell[CellGroupData[{
Cell[11216, 431, 415, 12, 40, 26, 0, "StyleData", "Message", "All",
  CounterIncrements->"Message",
  StyleMenuListing->None],
Cell[11634, 445, 123, 2, 70, 42, 0, "StyleData", "Message", "Presentation"],
Cell[11760, 449, 97, 1, 70, 39, 0, "StyleData", "Message", "Condensed"],
Cell[11860, 452, 123, 2, 70, 38, 0, "StyleData", "Message", "Printout"]
}, Closed]],

Cell[CellGroupData[{
Cell[12020, 459, 412, 12, 40, 24, 0, "StyleData", "Print", "All",
  CounterIncrements->"Print",
  StyleMenuListing->None],
Cell[12435, 473, 121, 2, 70, 40, 0, "StyleData", "Print", "Presentation"],
Cell[12559, 477, 95, 1, 70, 37, 0, "StyleData", "Print", "Condensed"],
Cell[12657, 480, 94, 1, 70, 36, 0, "StyleData", "Print", "Printout"]
}, Closed]],

Cell[CellGroupData[{
Cell[12788, 486, 389, 11, 40, 27, 0, "StyleData", "Graphics", "All",
  CounterIncrements->"Graphics",
  StyleMenuListing->None],
Cell[13180, 499, 101, 1, 70, 43, 0, "StyleData", "Graphics", "Presentation"],
Cell[13284, 502, 123, 2, 70, 40, 0, "StyleData", "Graphics", "Condensed"],
Cell[13410, 506, 122, 2, 70, 39, 0, "StyleData", "Graphics", "Printout"]
}, Closed]],

Cell[CellGroupData[{
Cell[13569, 513, 171, 5, 39, 28, 0, "StyleData", "CellLabel", "All",
  StyleMenuListing->None],
Cell[13743, 520, 60, 1, 70, 44, 0, "StyleData", "CellLabel", "Presentation"],
Cell[13806, 523, 56, 1, 70, 41, 0, "StyleData", "CellLabel", "Condensed"],
Cell[13865, 526, 82, 2, 70, 40, 0, "StyleData", "CellLabel", "Printout"]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{
Cell[13996, 534, 32, 0, 33, "Section"],

Cell[CellGroupData[{
Cell[14053, 538, 125, 4, 70, 25, 0, "StyleData", "Author", "All"],
Cell[14181, 544, 145, 4, 70, 41, 0, "StyleData", "Author", "Presentation"],
Cell[14329, 550, 139, 4, 70, 38, 0, "StyleData", "Author", "Condensed"],
Cell[14471, 556, 115, 3, 70, 37, 0, "StyleData", "Author", "Printout"]
}, Closed]],

Cell[CellGroupData[{
Cell[14623, 564, 132, 4, 70, 28, 0, "StyleData", "Copyright", "All"],
Cell[14758, 570, 119, 3, 70, 44, 0, "StyleData", "Copyright", 
"Presentation"],
Cell[14880, 575, 114, 3, 70, 41, 0, "StyleData", "Copyright", "Condensed"],
Cell[14997, 580, 89, 2, 70, 40, 0, "StyleData", "Copyright", "Printout"]
}, Closed]],

Cell[CellGroupData[{
Cell[15123, 587, 191, 6, 70, 26, 0, "StyleData", "Example", "All",
  CounterIncrements->"Example"],
Cell[15317, 595, 130, 3, 70, 42, 0, "StyleData", "Example", "Presentation"],
Cell[15450, 600, 124, 3, 70, 39, 0, "StyleData", "Example", "Condensed"],
Cell[15577, 605, 100, 2, 70, 38, 0, "StyleData", "Example", "Printout"]
}, Closed]],

Cell[CellGroupData[{
Cell[15714, 612, 405, 11, 70, 29, 0, "StyleData", "Definition", "All",
  CounterIncrements->"Definition"],
Cell[16122, 625, 111, 2, 70, 45, 0, "StyleData", "Definition", 
"Presentation"],
Cell[16236, 629, 77, 1, 70, 42, 0, "StyleData", "Definition", "Condensed"],
Cell[16316, 632, 105, 2, 70, 41, 0, "StyleData", "Definition", "Printout"]
}, Closed]],

Cell[CellGroupData[{
Cell[16458, 639, 403, 11, 70, 26, 0, "StyleData", "Theorem", "All",
  CounterIncrements->"Theorem"],
Cell[16864, 652, 108, 2, 70, 42, 0, "StyleData", "Theorem", "Presentation"],
Cell[16975, 656, 74, 1, 70, 39, 0, "StyleData", "Theorem", "Condensed"],
Cell[17052, 659, 102, 2, 70, 38, 0, "StyleData", "Theorem", "Printout"]
}, Closed]],

Cell[CellGroupData[{
Cell[17191, 666, 431, 11, 70, 31, 0, "StyleData", "ExerciseMain", "All",
  CounterIncrements->"Subsection",
  CounterAssignments->{{"Subsubsection", 0}}],
Cell[17625, 679, 151, 4, 70, 47, 0, "StyleData", "ExerciseMain", 
"Presentation"],
Cell[17779, 685, 102, 2, 70, 44, 0, "StyleData", "ExerciseMain", 
"Condensed"],
Cell[17884, 689, 184, 5, 70, 43, 0, "StyleData", "ExerciseMain", "Printout"]
}, Closed]],

Cell[CellGroupData[{
Cell[18105, 699, 332, 9, 70, 27, 0, "StyleData", "Exercise", "All",
  CounterIncrements->"Subsubsection"],
Cell[18440, 710, 118, 3, 70, 43, 0, "StyleData", "Exercise", "Presentation"],
Cell[18561, 715, 99, 2, 70, 40, 0, "StyleData", "Exercise", "Condensed"],
Cell[18663, 719, 170, 5, 70, 39, 0, "StyleData", "Exercise", "Printout"]
}, Closed]],

Cell[CellGroupData[{
Cell[18870, 729, 178, 5, 70, 31, 0, "StyleData", "ExerciseText", "All",
  CounterIncrements->"ExcersiceText"],
Cell[19051, 736, 134, 3, 70, 47, 0, "StyleData", "ExerciseText", 
"Presentation"],
Cell[19188, 741, 130, 3, 70, 44, 0, "StyleData", "ExerciseText", 
"Condensed"],
Cell[19321, 746, 105, 2, 70, 43, 0, "StyleData", "ExerciseText", "Printout"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[19475, 754, 43, 0, 33, "Section"],

Cell[CellGroupData[{
Cell[19543, 758, 250, 6, 70, 33, 0, "StyleData", "DisplayFormula", "All"],
Cell[19796, 766, 101, 2, 70, 49, 0, "StyleData", "DisplayFormula", 
"Presentation"],
Cell[19900, 770, 98, 2, 70, 46, 0, "StyleData", "DisplayFormula", 
"Condensed"],
Cell[20001, 774, 97, 2, 70, 45, 0, "StyleData", "DisplayFormula", "Printout"]
}, Closed]],

Cell[CellGroupData[{
Cell[20135, 781, 251, 7, 70, 34, 0, "StyleData", "ChemicalFormula", "All"],
Cell[20389, 790, 102, 2, 70, 50, 0, "StyleData", "ChemicalFormula", 
"Presentation"],
Cell[20494, 794, 99, 2, 70, 47, 0, "StyleData", "ChemicalFormula", 
"Condensed"],
Cell[20596, 798, 98, 2, 70, 46, 0, "StyleData", "ChemicalFormula", 
"Printout"]
}, Closed]],

Cell[CellGroupData[{
Cell[20731, 805, 94, 2, 70, 26, 0, "StyleData", "Program", "All"],
Cell[20828, 809, 102, 2, 70, 42, 0, "StyleData", "Program", "Presentation"],
Cell[20933, 813, 98, 2, 70, 39, 0, "StyleData", "Program", "Condensed"],
Cell[21034, 817, 96, 2, 70, 38, 0, "StyleData", "Program", "Printout"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[21179, 825, 49, 0, 33, "Section"],
Cell[21231, 827, 225, 4, 70, "Text"],

Cell[CellGroupData[{
Cell[21481, 835, 334, 7, 70, 35, 0, "StyleData", "NumberedEquation", "All",
  CounterIncrements->"NumberedEquation"],
Cell[21818, 844, 102, 1, 70, 51, 0, "StyleData", "NumberedEquation", 
"Presentation"],
Cell[21923, 847, 99, 1, 70, 48, 0, "StyleData", "NumberedEquation", 
"Condensed"],
Cell[22025, 850, 97, 1, 70, 47, 0, "StyleData", "NumberedEquation", 
"Printout"]
}, Closed]],

Cell[CellGroupData[{
Cell[22159, 856, 336, 7, 70, 33, 0, "StyleData", "NumberedFigure", "All",
  CounterIncrements->"NumberedFigure"],
Cell[22498, 865, 49, 0, 70, 49, 0, "StyleData", "NumberedFigure", 
"Presentation"],
Cell[22550, 867, 46, 0, 70, 46, 0, "StyleData", "NumberedFigure", 
"Condensed"],
Cell[22599, 869, 45, 0, 70, 45, 0, "StyleData", "NumberedFigure", "Printout"]
}, Closed]],

Cell[CellGroupData[{
Cell[22681, 874, 306, 7, 70, 32, 0, "StyleData", "NumberedTable", "All",
  CounterIncrements->"NumberedTable"],
Cell[22990, 883, 48, 0, 70, 48, 0, "StyleData", "NumberedTable", 
"Presentation"],
Cell[23041, 885, 45, 0, 70, 45, 0, "StyleData", "NumberedTable", 
"Condensed"],
Cell[23089, 887, 44, 0, 70, 44, 0, "StyleData", "NumberedTable", "Printout"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[23182, 893, 49, 0, 33, "Section"],
Cell[23234, 895, 149, 5, 70, 25, 0, "StyleData", "Header", "All",
  StyleMenuListing->None],
Cell[23386, 902, 126, 4, 70, 25, 0, "StyleData", "Footer", "All",
  StyleMenuListing->None],
Cell[23515, 908, 152, 5, 70, 29, 0, "StyleData", "PageNumber", "All",
  StyleMenuListing->None]
}, Closed]],

Cell[CellGroupData[{
Cell[23704, 918, 35, 0, 33, "Section"],
Cell[23742, 920, 184, 4, 70, "Text"],

Cell[CellGroupData[{
Cell[23951, 928, 316, 9, 70, 28, 0, "StyleData", "Hyperlink", "All",
  StyleMenuListing->None,
  ButtonStyleMenuListing->Automatic],
Cell[24270, 939, 44, 0, 70, 44, 0, "StyleData", "Hyperlink", "Presentation"],
Cell[24317, 941, 41, 0, 70, 41, 0, "StyleData", "Hyperlink", "Condensed"],
Cell[24361, 943, 95, 2, 70, 40, 0, "StyleData", "Hyperlink", "Printout"]
}, Closed]],
Cell[24471, 948, 104, 3, 70, "Text"],

Cell[CellGroupData[{
Cell[24600, 955, 307, 8, 70, 31, 0, "StyleData", "MainBookLink", "All",
  StyleMenuListing->None,
  ButtonStyleMenuListing->Automatic],
Cell[24910, 965, 47, 0, 70, 47, 0, "StyleData", "MainBookLink", 
"Presentation"],
Cell[24960, 967, 44, 0, 70, 44, 0, "StyleData", "MainBookLink", "Condensed"],
Cell[25007, 969, 98, 2, 70, 43, 0, "StyleData", "MainBookLink", "Printout"]
}, Closed]],

Cell[CellGroupData[{
Cell[25142, 976, 328, 9, 70, 29, 0, "StyleData", "AddOnsLink", "All",
  StyleMenuListing->None,
  ButtonStyleMenuListing->Automatic],
Cell[25473, 987, 45, 0, 70, 45, 0, "StyleData", "AddOnsLink", 
"Presentation"],
Cell[25521, 989, 42, 0, 70, 42, 0, "StyleData", "AddOnsLink", "Condensed"],
Cell[25566, 991, 95, 2, 70, 40, 0, "StyleData", "AddOnLink", "Printout"]
}, Closed]],

Cell[CellGroupData[{
Cell[25698, 998, 336, 9, 70, 31, 0, "StyleData", "RefGuideLink", "All",
  StyleMenuListing->None,
  ButtonStyleMenuListing->Automatic],
Cell[26037, 1009, 47, 0, 70, 47, 0, "StyleData", "RefGuideLink", 
"Presentation"],
Cell[26087, 1011, 44, 0, 70, 44, 0, "StyleData", "RefGuideLink", 
"Condensed"],
Cell[26134, 1013, 98, 2, 70, 43, 0, "StyleData", "RefGuideLink", "Printout"]
}, Closed]],

Cell[CellGroupData[{
Cell[26269, 1020, 319, 8, 70, 37, 0, "StyleData", "GettingStartedLink", 
"All",
  StyleMenuListing->None,
  ButtonStyleMenuListing->Automatic],
Cell[26591, 1030, 53, 0, 70, 53, 0, "StyleData", "GettingStartedLink", 
"Presentation"],
Cell[26647, 1032, 50, 0, 70, 50, 0, "StyleData", "GettingStartedLink", 
"Condensed"],
Cell[26700, 1034, 104, 2, 70, 49, 0, "StyleData", "GettingStartedLink", 
"Printout"]
}, Closed]],

Cell[CellGroupData[{
Cell[26841, 1041, 323, 8, 70, 39, 0, "StyleData", "OtherInformationLink", 
"All",
  StyleMenuListing->None,
  ButtonStyleMenuListing->Automatic],
Cell[27167, 1051, 55, 0, 70, 55, 0, "StyleData", "OtherInformationLink", 
"Presentation"],
Cell[27225, 1053, 52, 0, 70, 52, 0, "StyleData", "OtherInformationLink", 
"Condensed"],
Cell[27280, 1055, 106, 2, 70, 51, 0, "StyleData", "OtherInformationLink", 
"Printout"]
}, Closed]]
}, Closed]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

